/*************************************************************************
 *  File: act.skills.c                                  Land of Dangun    *
 *  Usage: Functions to handle skills  based on socials                    *
 *                                                                         *
 *  All rights reserved.  See license.doc for complete information.        *
 *                                                                         *
 *  Copyright (C) 1993, 94 by the Trustees of the Johns Hopkins University *
 *  DangunMUD is based on CircleMUD, Copyright (C) 1990, 1991.             *
 ************************************************************************ */

#include "conf.h"
#include "sysdep.h"


#include "structs.h"
#include "utils.h"
#include "comm.h"
#include "interpreter.h"
#include "handler.h"
#include "db.h"
#include "spells.h"


/* globals */
extern int top_of_skillt;
extern struct skill_messg *ski_mess_list;

/* local functions */
char *fread_skills(FILE *fl, int nr);
int find_skill(int cmd);
ACMD(do_skills);
//ACMD(do_insult);
void boot_skill_messages(void);
void free_skill_messages(void);
void free_skills(struct skill_messg *mess);
void free_command_list(void);

ACMD(do_skills)
{
	char arg[MAX_INPUT_LENGTH], part[MAX_INPUT_LENGTH];
	int act_nr;
	struct skill_messg *skills;
	struct char_data *vict;
	struct obj_data *targ;

	if ((act_nr = find_skill(cmd)) < 0)
	{
		send_to_char(ch, "׷ ൿ  ʽϴ.\r\n");
		return;
	}

	skills = &ski_mess_list[act_nr];

	if (!argument || !*argument)
	{
		send_to_char(ch, "%s\r\n", skills->char_no_arg);
		act(skills->others_no_arg, skills->hide, ch, 0, 0, TO_ROOM);
		return;
	}

	two_arguments(argument, arg, part);

	if ((!skills->char_body_found) && (*part))
	{
		send_to_char(ch, " ൿ  ʴ Դϴ.\r\n");
		return;
	}

	if (!skills->char_found) 
		*arg = '\0';

	if (skills->char_found && argument)
		one_argument(argument, arg);
	else
		*arg = '\0';

	vict = get_char_vis(ch, arg, NULL, FIND_CHAR_ROOM);
	if (!vict)
	{
		if (skills->char_obj_found)
		{
			targ = get_obj_in_list_vis(ch, arg, NULL, ch->carrying);
			if (!targ) targ = get_obj_in_list_vis(ch, arg, NULL, world[ch->in_room].contents);
			if (targ)
			{
				act(skills->char_obj_found, skills->hide, ch, targ, 0, TO_CHAR);
				act(skills->others_obj_found, skills->hide, ch, targ, 0, TO_ROOM);
				return;
			}
		}
		if (skills->not_found)
			send_to_char(ch, "%s\r\n", skills->not_found);
		else
			send_to_char(ch, "I don't see anything by that name here.\r\n");
		return;
	} 

	if (vict == ch)
	{
		if (skills->char_auto)
			send_to_char(ch, "%s\r\n", skills->char_auto);
		else
			send_to_char(ch, "Erm, no.");
		act(skills->others_auto, skills->hide, ch, 0, 0, TO_ROOM);
		return;
	} 

	if (GET_POS(vict) < skills->min_victim_position)
		act("$N is not in a proper position for that.", FALSE, ch, 0, vict, TO_CHAR | TO_SLEEP);
	else
	{
		if (*part)
		{
			act(skills->char_body_found, 0, ch, (struct obj_data *)part, vict, TO_CHAR | TO_SLEEP);
			act(skills->others_body_found, skills->hide, ch, (struct obj_data *)part, vict, TO_NOTVICT);
			act(skills->vict_body_found, skills->hide, ch, (struct obj_data *)part, vict, TO_VICT);
		}
		else
		{
			act(skills->char_found, 0, ch, 0, vict, TO_CHAR | TO_SLEEP);
			act(skills->others_found, skills->hide, ch, 0, vict, TO_NOTVICT);
			act(skills->vict_found, skills->hide, ch, 0, vict, TO_VICT);
		}
	}
}

void boot_skill_messages(void)
{
	FILE *fl;
	int nr = 0, hide, min_char_pos, min_pos, min_lvl, turn_num, curr_ski = -1;
	char next_ski[MAX_STRING_LENGTH], sorted[MAX_INPUT_LENGTH];

/*	if (CONFIG_NEW_SKILLS == TRUE)
	{
		// open skill file */
		if (!(fl = fopen(SKILL_FILE, "r")))
		{
			log("SYSERR:    ϴ. '%s': %s", 
					SKILL_FILE, strerror(errno));
			exit(1);
		}
		/* count skills */
		*next_ski = '\0';
		while (!feof(fl))
		{
			fgets(next_ski, MAX_STRING_LENGTH, fl);
			if (*next_ski == '~') top_of_skillt++;
		}
/*	}
	else
	{ // old style

		// open skill file
		if (!(fl = fopen(SKILL_FILE, "r")))
		{
			log("SYSERR: can't open skills file '%s': %s", SKILL_FILE, strerror(errno));
			exit(1);
		}
		// count skills
		while (!feof(fl))
		{
			fgets(next_ski, MAX_STRING_LENGTH, fl);
			if (*next_ski == '\n' || *next_ski == '\r') top_of_skillt++; //all skills are followed by a blank line
		}
	} */

	log("Skills table contains %d skills.", top_of_skillt);
	rewind(fl);

	CREATE(ski_mess_list, struct skill_messg, top_of_skillt + 1);

	/* now read 'em */
	for (;;)
	{
		fscanf(fl, " %s ", next_ski);
		if (*next_ski == '$') break;

/*		if (CONFIG_NEW_SKILLS == TRUE)
		{ */
			if (fscanf(fl, " %s %d %d %d %d %d\n", 
						sorted, &hide, &min_char_pos, &min_pos, &min_lvl, &turn_num) != 6)
			{
				log("SYSERR: format error in skill file near skill '%s'", next_ski);
				exit(1);
			}
			curr_ski++;
			ski_mess_list[curr_ski].command = strdup(next_ski+1);
			ski_mess_list[curr_ski].sort_as = strdup(sorted);
			ski_mess_list[curr_ski].hide = hide;
			ski_mess_list[curr_ski].min_char_position = min_char_pos;
			ski_mess_list[curr_ski].min_victim_position = min_pos;
			ski_mess_list[curr_ski].min_level_char = min_lvl;
			ski_mess_list[curr_ski].turnnum = turn_num;
/*		}
		else
		{  // old style
			if (fscanf(fl, " %d %d \n", &hide, &min_pos) != 2)
			{
				log("SYSERR: format error in skill file near skill '%s'", next_ski);
				exit(1);
			}
			curr_ski++;
			ski_mess_list[curr_ski].command = strdup(next_ski);
			ski_mess_list[curr_ski].sort_as = strdup(next_ski);
			ski_mess_list[curr_ski].hide = hide;
			ski_mess_list[curr_ski].min_char_position = POS_RESTING;
			ski_mess_list[curr_ski].min_victim_position = min_pos;
			ski_mess_list[curr_ski].min_level_char = 0;
			ski_mess_list[curr_ski].turnnum = turn_num;

		}*/

#ifdef CIRCLE_ACORN
		if (fgetc(fl) != '\n')
			log("SYSERR: Acorn bug workaround failed.");
#endif

		ski_mess_list[curr_ski].char_no_arg = fread_skills(fl, nr);
		ski_mess_list[curr_ski].others_no_arg = fread_skills(fl, nr);
		ski_mess_list[curr_ski].char_found = fread_skills(fl, nr);

		/* if no char_found, the rest is to be ignored */
		if (!ski_mess_list[curr_ski].char_found)
			continue;

		ski_mess_list[curr_ski].others_found = fread_skills(fl, nr);
		ski_mess_list[curr_ski].vict_found = fread_skills(fl, nr);
		ski_mess_list[curr_ski].not_found = fread_skills(fl, nr);
		ski_mess_list[curr_ski].char_auto = fread_skills(fl, nr);
		ski_mess_list[curr_ski].others_auto = fread_skills(fl, nr);

		/*if (CONFIG_NEW_SKILLS == FALSE) 
			continue;*/

		ski_mess_list[curr_ski].char_body_found = fread_skills(fl, nr);
		ski_mess_list[curr_ski].others_body_found = fread_skills(fl, nr);
		ski_mess_list[curr_ski].vict_body_found = fread_skills(fl, nr);
		ski_mess_list[curr_ski].char_obj_found = fread_skills(fl, nr);
		ski_mess_list[curr_ski].others_obj_found = fread_skills(fl, nr);
	}

	/* close file & set top */
	fclose(fl);
	assert(curr_ski <= top_of_skillt);
	top_of_skillt = curr_ski;
}

char *fread_skills(FILE *fl, int nr)
{
	char buf[MAX_STRING_LENGTH];

	fgets(buf, MAX_STRING_LENGTH, fl);
	if (feof(fl))
	{
		log("SYSERR: fread_skills: unexpected EOF near skills #%d", nr);
		exit(1);
	}
	if (*buf == '#')
		return (NULL);

	buf[strlen(buf) - 1] = '\0';
	return (strdup(buf));
}

void free_skill_messages(void) 
{
	struct skill_messg *mess;
	int i;

	for (i = 0;i <= top_of_skillt;i++) 
	{
		mess = &ski_mess_list[i];
		free_skills(mess);
	}
	free(ski_mess_list);

}  

void free_skills(struct skill_messg *mess) 
{
	if (mess->command) free(mess->command);
	if (mess->sort_as) free(mess->sort_as);
	if (mess->char_no_arg) free(mess->char_no_arg);
	if (mess->others_no_arg) free(mess->others_no_arg);
	if (mess->char_found) free(mess->char_found);
	if (mess->others_found) free(mess->others_found);
	if (mess->vict_found) free(mess->vict_found);
	if (mess->char_body_found) free(mess->char_body_found);
	if (mess->others_body_found) free(mess->others_body_found);
	if (mess->vict_body_found) free(mess->vict_body_found);
	if (mess->not_found) free(mess->not_found);
	if (mess->char_auto) free(mess->char_auto);
	if (mess->others_auto) free(mess->others_auto);
	if (mess->char_obj_found) free(mess->char_obj_found);
	if (mess->others_obj_found) free(mess->others_obj_found);
	memset(mess, 0, sizeof(struct skill_messg));
}

int find_skill(int cmd)
{
	int bot, top, mid;

	bot = 0;
	top = top_of_skillt;

	if (top < 0)
		return (-1);

	for (;;)
	{
		mid = (bot + top) / 2;

		if (ski_mess_list[mid].act_nr == cmd)
			return (mid);
		if (bot >= top)
			return (-1);

		if (ski_mess_list[mid].act_nr > cmd)
			top = --mid;
		else
			bot = ++mid;
	}
}
