/*
 * File: skedit.c
 * Comment: OLC for MUDs -- this one edits skills
 * by MiRR 05/05/04
 * for use with OasisOLC
 * based on socials
 */

#include "conf.h"
#include "sysdep.h"

#include "structs.h"
#include "interpreter.h"
#include "handler.h"
#include "comm.h"
#include "utils.h"
#include "db.h"
#include "oasis.h"
#include "screen.h"
#include "constants.h"
#include "genolc.h"

/* external functs */
int sort_command_helper(const void *a, const void *b);
void sort_commands(void);
void create_command_list(void);
void free_skills(struct skill_messg *skills);

/* function protos */
void skedit_disp_menu(struct descriptor_data * d);
void skedit_parse(struct descriptor_data * d, char *arg, char *arg1);
void skedit_setup_new(struct descriptor_data *d);
void skedit_setup_existing(struct descriptor_data *d, int real_num);
void skedit_save_to_disk(struct descriptor_data *d);
void skedit_save_internally(struct descriptor_data *d);
int skedit_find_command(const char *txt);
void disp_tdam(struct descriptor_data * d);
void skedit_tdam(struct descriptor_data * d, char* val, char* arg);
void skedit_turndam(struct descriptor_data * d, char* arg, char* arg1);
/*
 * Utils and exported functions.
 */

/*------------------------------------------------------------------------*\
  Utils and exported functions.
  \*------------------------------------------------------------------------*/

ACMD(do_oasis_skedit)
{
	char arg[MAX_INPUT_LENGTH];
	struct descriptor_data *d;
	int i;

/*	if (CONFIG_NEW_SKILLS == 0)
	{
		send_to_char(ch, " Ұ մϴ.\r\n");
		return;
	}
*/
	if (GET_OLC_ZONE(ch) != SKEDIT_PERMISSION && GET_RACE(ch) < LVL_IMPL)
	{
		send_to_char(ch, "   ϴ.\r\n");
		return;
	}

	for (d = descriptor_list; d; d = d->next)
		if (STATE(d) == CON_SKEDIT)
		{
			send_to_char(ch, " Դϴ.\r\n");
			return;
		}

	one_argument(argument, arg);

	if (!*arg)
	{
		send_to_char(ch, "  Է ּ.\r\n");
		return;
	}

	d = ch->desc;

	if (!str_cmp("", arg))
	{
		mudlog(CMP, MAX(LVL_BUILDER, GET_INVIS_LEV(ch)), TRUE, "OLC: %s saves skills.", GET_NAME(ch));
		send_to_char(ch, " մϴ..\r\n");
		skedit_save_to_disk(d);
		send_to_char(ch, "Ϸ.\r\n");
		return;
	}

	/*
	 * Give descriptor an OLC structure.
	 */
	if (d->olc)
	{
		mudlog(BRF, LVL_IMMORT, TRUE, "SYSERR: do_oasis: Player already had olc structure.");
		free(d->olc);
	}
	CREATE(d->olc, struct oasis_olc_data, 1);

	OLC_NUM(d) = 0;
	OLC_STORAGE(d) = strdup(arg);

	for (OLC_ZNUM(d) = 0; (OLC_ZNUM(d) <= top_of_skillt); OLC_ZNUM(d)++)  
		if (is_abbrev(OLC_STORAGE(d), ski_mess_list[OLC_ZNUM(d)].command))
			break;

	if (OLC_ZNUM(d) > top_of_skillt)  {
		if ((i = skedit_find_command(OLC_STORAGE(d))) != -1)  {
			send_to_char(ch, "'%s' ̹ մϴ. (%s).\r\n", OLC_STORAGE(d), complete_cmd_info[i].command);
			cleanup_olc(d, CLEANUP_ALL);
			return;
		}
		send_to_char(ch, "'%s'  ߰ Űڽϱ? ", OLC_STORAGE(d));
		OLC_MODE(d) = SKEDIT_CONFIRM_ADD;
	}
	else
	{
		send_to_char(ch, "'%s'  մϱ? ", ski_mess_list[OLC_ZNUM(d)].command);
		OLC_MODE(d) = SKEDIT_CONFIRM_EDIT;
	}
	STATE(d) = CON_SKEDIT;
	act("$nԲ OLC մϴ.", TRUE, d->character, 0, 0, TO_ROOM);
	SET_BIT_AR(PLR_FLAGS(ch), PLR_WRITING);
	mudlog(CMP, LVL_IMMORT, TRUE, "OLC: %s  .", GET_NAME(ch));
}


void skedit_setup_new(struct descriptor_data *d)
{
//	int i;
	CREATE(OLC_SKILL(d), struct skill_messg, 1);
	OLC_SKILL(d)->command             = strdup(OLC_STORAGE(d));
	OLC_SKILL(d)->sort_as             = strdup(OLC_STORAGE(d));
	OLC_SKILL(d)->hide                = 0;
	OLC_SKILL(d)->min_victim_position = POS_STANDING;
	OLC_SKILL(d)->min_char_position   = POS_STANDING;
	OLC_SKILL(d)->min_level_char      = 0;
	OLC_SKILL(d)->turnnum			  = 0;
	OLC_SKILL(d)->tdam[0]			  = 0;
	OLC_SKILL(d)->dam				  = 0;
	OLC_SKILL(d)->char_no_arg         = strdup("This skill is unfinished.");
	OLC_SKILL(d)->others_no_arg       = strdup("This skill is unfinished.");
	OLC_SKILL(d)->char_found          = NULL;
	OLC_SKILL(d)->others_found        = NULL;
	OLC_SKILL(d)->vict_found          = NULL;
	OLC_SKILL(d)->not_found           = NULL;
	OLC_SKILL(d)->char_auto           = NULL;
	OLC_SKILL(d)->others_auto         = NULL;
	OLC_SKILL(d)->char_body_found     = NULL;
	OLC_SKILL(d)->others_body_found   = NULL;
	OLC_SKILL(d)->vict_body_found     = NULL;
	OLC_SKILL(d)->char_obj_found      = NULL;
	OLC_SKILL(d)->others_obj_found    = NULL;
	skedit_disp_menu(d);
	OLC_VAL(d) = 0;
}

/*------------------------------------------------------------------------*/

void skedit_setup_existing(struct descriptor_data *d, int real_num)
{
	CREATE(OLC_SKILL(d), struct skill_messg, 1);

	OLC_SKILL(d)->command             = strdup(ski_mess_list[real_num].command);
	OLC_SKILL(d)->sort_as             = strdup(ski_mess_list[real_num].sort_as);
	OLC_SKILL(d)->hide                = ski_mess_list[real_num].hide;
	OLC_SKILL(d)->min_victim_position = ski_mess_list[real_num].min_victim_position;
	OLC_SKILL(d)->min_char_position   = ski_mess_list[real_num].min_char_position;
	OLC_SKILL(d)->min_level_char      = ski_mess_list[real_num].min_level_char;
	OLC_SKILL(d)->turnnum			  = ski_mess_list[real_num].turnnum;
	OLC_SKILL(d)->tdam[4]			  = ski_mess_list[real_num].tdam[4];
	OLC_SKILL(d)->dam				  = ski_mess_list[real_num].dam;

	if (ski_mess_list[real_num].char_no_arg)
		OLC_SKILL(d)->char_no_arg       = strdup(ski_mess_list[real_num].char_no_arg);

	if (ski_mess_list[real_num].others_no_arg)
		OLC_SKILL(d)->others_no_arg     = strdup(ski_mess_list[real_num].others_no_arg);

	if (ski_mess_list[real_num].char_found)
		OLC_SKILL(d)->char_found        = strdup(ski_mess_list[real_num].char_found);

	if (ski_mess_list[real_num].others_found)
		OLC_SKILL(d)->others_found      = strdup(ski_mess_list[real_num].others_found);

	if (ski_mess_list[real_num].vict_found)
		OLC_SKILL(d)->vict_found        = strdup(ski_mess_list[real_num].vict_found);

	if (ski_mess_list[real_num].not_found)
		OLC_SKILL(d)->not_found         = strdup(ski_mess_list[real_num].not_found);

	if (ski_mess_list[real_num].char_auto)
		OLC_SKILL(d)->char_auto         = strdup(ski_mess_list[real_num].char_auto);

	if (ski_mess_list[real_num].others_auto)
		OLC_SKILL(d)->others_auto       = strdup(ski_mess_list[real_num].others_auto);

	if (ski_mess_list[real_num].char_body_found)
		OLC_SKILL(d)->char_body_found   = strdup(ski_mess_list[real_num].char_body_found);

	if (ski_mess_list[real_num].others_body_found)
		OLC_SKILL(d)->others_body_found = strdup(ski_mess_list[real_num].others_body_found);

	if (ski_mess_list[real_num].vict_body_found)
		OLC_SKILL(d)->vict_body_found   = strdup(ski_mess_list[real_num].vict_body_found);

	if (ski_mess_list[real_num].char_obj_found)
		OLC_SKILL(d)->char_obj_found    = strdup(ski_mess_list[real_num].char_obj_found);

	if (ski_mess_list[real_num].others_obj_found)
		OLC_SKILL(d)->others_obj_found  = strdup(ski_mess_list[real_num].others_obj_found);

	OLC_VAL(d) = 0;
	skedit_disp_menu(d);
}


void skedit_save_internally(struct descriptor_data *d)
{
	struct skill_messg *new_skill_mess_list = NULL;
	int i;

	/* add a new skill into the list */
	if (OLC_ZNUM(d) > top_of_skillt)
	{
		CREATE(new_skill_mess_list, struct skill_messg, top_of_skillt + 2);
		for (i = 0; i <= top_of_skillt; i++)
			new_skill_mess_list[i] = ski_mess_list[i];
		new_skill_mess_list[++top_of_skillt] = *OLC_SKILL(d);
		free(ski_mess_list);
		ski_mess_list = new_skill_mess_list;
	}
	/* pass the editted action back to the list - no need to add */
	else
	{
		i = skedit_find_command(OLC_SKILL(d)->command);
		OLC_SKILL(d)->act_nr = ski_mess_list[OLC_ZNUM(d)].act_nr;
		/* why did i do this..? hrm */
		free_skills(ski_mess_list + OLC_ZNUM(d));
		ski_mess_list[OLC_ZNUM(d)] = *OLC_SKILL(d);
	}

	create_command_list();
	sort_commands();

	add_to_save_list(SKEDIT_PERMISSION, SL_SKILL);
	skedit_save_to_disk(d); /* autosave by Rumble */
}


/*------------------------------------------------------------------------*/
void skedit_save_to_disk(struct descriptor_data *d)
{
	FILE *fp;
	int i;
	if (!(fp = fopen(SKILL_FILE, "w+"))) 
	{
		char error[MAX_STRING_LENGTH];
		snprintf(error, sizeof(error), "Can't open skills file '%s'", SKILL_FILE);
		perror(error);
		exit(1);
	}

	for (i = 0; i <= top_of_skillt; i++)  
	{
		fprintf(fp, "~%s %s %d %d %d %d %d %d %d\n",
				ski_mess_list[i].command,
				ski_mess_list[i].sort_as,
				ski_mess_list[i].hide,
				ski_mess_list[i].min_char_position,
				ski_mess_list[i].min_victim_position,
				ski_mess_list[i].min_level_char,
				ski_mess_list[i].turnnum, 
				ski_mess_list[i].tdam[0],
				ski_mess_list[i].tdam[1],
				ski_mess_list[i].tdam[2],
				ski_mess_list[i].tdam[3],
				ski_mess_list[i].tdam[4],
				ski_mess_list[i].dam);
		fprintf(fp, "%s\n%s\n%s\n%s\n",
				((ski_mess_list[i].char_no_arg)?ski_mess_list[i].char_no_arg:"#"),
				((ski_mess_list[i].others_no_arg)?ski_mess_list[i].others_no_arg:"#"),
				((ski_mess_list[i].char_found)?ski_mess_list[i].char_found:"#"),
				((ski_mess_list[i].others_found)?ski_mess_list[i].others_found:"#"));
		fprintf(fp, "%s\n%s\n%s\n%s\n",
				((ski_mess_list[i].vict_found)?ski_mess_list[i].vict_found:"#"),
				((ski_mess_list[i].not_found)?ski_mess_list[i].not_found:"#"),
				((ski_mess_list[i].char_auto)?ski_mess_list[i].char_auto:"#"),
				((ski_mess_list[i].others_auto)?ski_mess_list[i].others_auto:"#"));
		fprintf(fp, "%s\n%s\n%s\n",
				((ski_mess_list[i].char_body_found)?ski_mess_list[i].char_body_found:"#"),
				((ski_mess_list[i].others_body_found)?ski_mess_list[i].others_body_found:"#"),
				((ski_mess_list[i].vict_body_found)?ski_mess_list[i].vict_body_found:"#"));
		fprintf(fp, "%s\n%s\n\n",
				((ski_mess_list[i].char_obj_found)?ski_mess_list[i].char_obj_found:"#"),
				((ski_mess_list[i].others_obj_found)?ski_mess_list[i].others_obj_found:"#"));
	}

	fprintf(fp, "$\n");
	fclose(fp);
	remove_from_save_list(SKEDIT_PERMISSION, SL_SKILL); 
}

/*------------------------------------------------------------------------*/

/* Menu functions */

void disp_tdam(struct descriptor_data * d)
{
	int i;
	for (i = 1; i <= OLC_SKILL(d)->turnnum; i++)
		write_to_output(d, "%s%d%s)%d : %s%d%s\r\n",
		grn, i, nrm, i, yel, OLC_SKILL(d)->tdam[i-1], nrm);

	write_to_output(d, " (: 0): ");

	OLC_MODE(d) = SKEDIT_TURN_DAM;
}

void skedit_tdam(struct descriptor_data * d, char* val, char *arg)
{
	int j = 0;

	write_to_output(d, "  Է: ");
	j = atoi(arg);
	
	if (j < 0 || j > 20000)
	{
		disp_tdam(d);
		return;
	}

	OLC_SKILL(d)->tdam[atoi(val)] = j;
	return;
}

		
/* the main menu */
void skedit_disp_menu(struct descriptor_data * d)
{
	struct skill_messg *skills = OLC_SKILL(d);
	struct char_data *ch        = d->character;

	get_char_colors(ch);

	write_to_output(d, 
			"%s--  \r\n"
			"%sn%s)          : %s%-15.15s%s %s1%s) ª̸  : %s%-15.15s%s\r\n"
			"%s2%s) ڼ[ڽ]: %s%-8.8s        %s3%s) ڼ []: %s%-8.8s\r\n"
			"%s4%s) [ڽ]: %s%-3d             %s5%s) Show if Invisible: %s%s\r\n"
			"%s6%s)   : %s%d    %s7%s) ϴ %s: %s%-6d   %s ѵ%s: %s%-6d%s\r\n"
			"%s8%s) Char    [NO ARG]: %s%s\r\n"
			"%s9%s) Others  [NO ARG]: %s%s\r\n"
			"%sj%s) Char [NOT FOUND]: %s%s\r\n"
			"%sa%s) Char  [ARG SELF]: %s%s\r\n"
			"%sb%s) Others[ARG SELF]: %s%s\r\n"
			"%sc%s) Char      [VICT]: %s%s\r\n"
			"%sd%s) Others    [VICT]: %s%s\r\n"
			"%se%s) Victim    [VICT]: %s%s\r\n"
			"%sf%s) Char  [BODY PRT]: %s%s\r\n"
			"%sg%s) Others[BODY PRT]: %s%s\r\n"
			"%sh%s) Victim[BODY PRT]: %s%s\r\n"
			"%sy%s) Char       [OBJ]: %s%s\r\n"
			"%sk%s) Others     [OBJ]: %s%s\r\n"
			"%sq%s) \r\n"
			":",
			nrm, grn, nrm,
			yel, skills->command, nrm,
			grn, nrm,
			yel, skills->sort_as, nrm,
			grn, nrm,
			cyn, position_types[skills->min_char_position],
			grn, nrm,
			cyn, position_types[skills->min_victim_position],
			grn, nrm,
			cyn, skills->min_level_char,
			grn, nrm,
			cyn, (skills->hide?"HIDDEN":"NOT HIDDEN"),
			grn, nrm, cyn,
			skills->turnnum, grn, nrm, cyn, grn, skills->tdam[0], grn, nrm, cyn,
			skills->dam, grn, nrm, cyn,
			skills->char_no_arg ? skills->char_no_arg : "<Null>",
			grn, nrm, cyn,
			skills->others_no_arg ? skills->others_no_arg : "<Null>",
			grn, nrm, cyn,
			skills->not_found ? skills->not_found : "<Null>",
			grn, nrm, cyn,
			skills->char_auto ? skills->char_auto : "<Null>",
			grn, nrm, cyn,
			skills->others_auto ? skills->others_auto : "<Null>",
			grn, nrm, cyn,
			skills->char_found ? skills->char_found : "<Null>",
			grn, nrm, cyn,
			skills->others_found ? skills->others_found : "<Null>",
			grn, nrm, cyn,
			skills->vict_found ? skills->vict_found : "<Null>",
			grn, nrm, cyn,
			skills->char_body_found ? skills->char_body_found : "<Null>",
			grn, nrm, cyn,
			skills->others_body_found ? skills->others_body_found : "<Null>",
			grn, nrm, cyn,
			skills->vict_body_found ? skills->vict_body_found : "<Null>",
			grn, nrm, cyn,
			skills->char_obj_found ? skills->char_obj_found : "<Null>",
			grn, nrm, cyn,
			skills->others_obj_found ? skills->others_obj_found : "<Null>",
			grn, nrm);

	OLC_MODE(d) = SKEDIT_MAIN_MENU;
}


/*
 * The main loop
 */
void skedit_parse(struct descriptor_data * d, char *arg, char *arg1)
{
	int i, j = 0;

	switch (OLC_MODE(d))
	{
		case SKEDIT_CONFIRM_SAVESTRING:
			switch (*arg)
			{
				case '':
				case 'Y':
				case 'y':
					skedit_save_internally(d);
					mudlog (CMP, LVL_IMPL, TRUE, "OLC: %s  %s ", 
							GET_NAME(d->character), OLC_SKILL(d)->command);

					/* do not free the strings.. just the structure */
					cleanup_olc(d, CLEANUP_STRUCTS);
					write_to_output(d, "մϴ.\r\n");
					break;
				case 'n': case 'N':
					/* free everything up, including strings etc */
					cleanup_olc(d, CLEANUP_ALL);
					break;
				default:
					write_to_output(d, "߸ ϼ̽ϴ!\r\n"
							"Ͻðڽϱ? ");
					break;
			}
			return; /* end of SKEDIT_CONFIRM_SAVESTRING */

		case SKEDIT_CONFIRM_EDIT:
			switch (*arg)  {
				case 'y': case 'Y': case '':
					skedit_setup_existing(d, OLC_ZNUM(d));
					break;
				case 'q': case 'Q':
					cleanup_olc(d, CLEANUP_ALL);
					break;
				case 'n': case 'N':
					OLC_ZNUM(d)++;
					for (;(OLC_ZNUM(d) <= top_of_skillt); OLC_ZNUM(d)++)
						if (is_abbrev(OLC_STORAGE(d), ski_mess_list[OLC_ZNUM(d)].command)) 
							break;

					if (OLC_ZNUM(d) > top_of_skillt)
					{
						if (skedit_find_command(OLC_STORAGE(d)) != -1)  {
							cleanup_olc(d, CLEANUP_ALL);
							break;
						}
						write_to_output(d, "'%s'  ߰Ͻðڽϱ? ",
								OLC_STORAGE(d));
						OLC_MODE(d) = SKEDIT_CONFIRM_ADD;
					}
					else  {
						write_to_output(d, "'%s'  Ͻðڽϱ? ",
								ski_mess_list[OLC_ZNUM(d)].command);
						OLC_MODE(d) = SKEDIT_CONFIRM_EDIT;
					}
					break;
				default:
					write_to_output(d, "߸ ϼ̽ϴ!\r\n"
							"'%s'  Ͻðڽϱ? ", 
							ski_mess_list[OLC_ZNUM(d)].command);
					break;
			}
			return;

		case SKEDIT_CONFIRM_ADD:
			switch (*arg)  {
				case 'y': case 'Y': case '':
					skedit_setup_new(d);
					break;
				case 'n': case 'N': case 'q': case 'Q':
					cleanup_olc(d, CLEANUP_ALL);
					break;
				default:
					write_to_output(d, "߸ ϼ̽ϴ!\r\n"
							"'%s'  ߰Ͻðڽϱ? ", 
							OLC_STORAGE(d));
					break;
			}
			return;

		case SKEDIT_MAIN_MENU:
			switch (*arg)
			{
				case 'q': case 'Q':
					if (OLC_VAL(d))  { /* Something was modified */
						write_to_output(d, "Ͻðڽϱ? ");
						OLC_MODE(d) = SKEDIT_CONFIRM_SAVESTRING;
					}
					else cleanup_olc(d, CLEANUP_ALL);
					break;
				case 'n':
					write_to_output(d, " ̸ Էϼ: ");
					OLC_MODE(d) = SKEDIT_ACTION_NAME;
					return;
				case '1':
					write_to_output(d, " ̸ Էϼ (ɸ ): ");
					OLC_MODE(d) = SKEDIT_SORT_AS;
					return;
				case '2':
					write_to_output(d, "  ִ ⺻ ڼ (ڽ):\r\n");
					{
						int i;
						for (i=POS_DEAD;i<=POS_STANDING;i++)
							write_to_output(d, "   %d) %s\r\n", i, position_types[i]);

						write_to_output(d, ": ");
					}
					OLC_MODE(d) = SKEDIT_MIN_CHAR_POS;
					return;
				case '3':
					write_to_output(d, " ⺻ ڼ:\r\n");
					{
						int i;
						for (i=POS_DEAD;i<=POS_STANDING;i++)
							write_to_output(d, "   %d) %s\r\n", i, position_types[i]);

						write_to_output(d, ": ");
					}
					OLC_MODE(d) = SKEDIT_MIN_VICT_POS;
					return;
				case '4':
					write_to_output(d, " ϱ  : ");
					OLC_MODE(d) = SKEDIT_MIN_CHAR_LEVEL;
					return;
				case '5':
					OLC_SKILL(d)->hide = !OLC_SKILL(d)->hide;
					skedit_disp_menu(d);
					OLC_VAL(d) = 1;
					break;
				case '6':
					write_to_output(d, "   : ");
					OLC_MODE(d) = SKEDIT_TURN;
					return;
				case '7':
					write_to_output(d, "ϴ  Է");
					OLC_MODE(d) = SKEDIT_TURN_DAM;
					return;					
				case '8':
					write_to_output(d, "Enter skill shown to the Character when there is no argument supplied.\r\n"
							"[OLD]: %s\r\n"
							"[NEW]: ",
							((OLC_SKILL(d)->char_no_arg)?OLC_SKILL(d)->char_no_arg:"NULL"));
					OLC_MODE(d) = SKEDIT_NOVICT_CHAR;
					return;
				case '9':
					write_to_output(d, "Enter skill shown to Others when there is no argument supplied.\r\n"
							"[OLD]: %s\r\n"
							"[NEW]: ",
							((OLC_SKILL(d)->others_no_arg)?OLC_SKILL(d)->others_no_arg:"NULL"));
					OLC_MODE(d) = SKEDIT_NOVICT_OTHERS;
					return;
				case 'j':
				case 'J':
					write_to_output(d, "Enter text shown to the Character when his victim isnt found.\r\n"
							"[OLD]: %s\r\n"
							"[NEW]: ",
							((OLC_SKILL(d)->not_found)?OLC_SKILL(d)->not_found:"NULL"));

					OLC_MODE(d) = SKEDIT_VICT_NOT_FOUND;
					return;
				case 'a':
				case 'A':
					write_to_output(d, "Enter skill shown to the Character when it is its own victim.\r\n"
							"[OLD]: %s\r\n"
							"[NEW]: ",
							((OLC_SKILL(d)->char_auto)?OLC_SKILL(d)->char_auto:"NULL"));

					OLC_MODE(d) = SKEDIT_SELF_CHAR;
					return;
				case 'b': case 'B':
					write_to_output(d, "Enter skill shown to Others when the Char is its own victim.\r\n"
							"[OLD]: %s\r\n"
							"[NEW]: ",
							((OLC_SKILL(d)->others_auto)?OLC_SKILL(d)->others_auto:"NULL"));

					OLC_MODE(d) = SKEDIT_SELF_OTHERS;
					return;
				case 'c': case 'C':
					write_to_output(d, "Enter normal skill shown to the Character when the victim is found.\r\n"
							"[OLD]: %s\r\n"
							"[NEW]: ",
							((OLC_SKILL(d)->char_found)?OLC_SKILL(d)->char_found:"NULL"));

					OLC_MODE(d) = SKEDIT_VICT_CHAR_FOUND;
					return;
				case 'd': case 'D':
					write_to_output(d, "Enter normal skill shown to Others when the victim is found.\r\n"
							"[OLD]: %s\r\n"
							"[NEW]: ",
							((OLC_SKILL(d)->others_found)?OLC_SKILL(d)->others_found:"NULL"));

					OLC_MODE(d) = SKEDIT_VICT_OTHERS_FOUND;
					return;
				case 'e': case 'E':
					write_to_output(d, "Enter normal skill shown to the Victim when the victim is found.\r\n"
							"[OLD]: %s\r\n"
							"[NEW]: ",
							((OLC_SKILL(d)->vict_found)?OLC_SKILL(d)->vict_found:"NULL"));

					OLC_MODE(d) = SKEDIT_VICT_VICT_FOUND;
					return;
				case 'f': case 'F':
					write_to_output(d, "Enter 'body part' skill shown to the Character when the victim is found.\r\n"
							"[OLD]: %s\r\n"
							"[NEW]: ",
							((OLC_SKILL(d)->char_body_found)?OLC_SKILL(d)->char_body_found:"NULL"));

					OLC_MODE(d) = SKEDIT_VICT_CHAR_BODY_FOUND;
					return;
				case 'g': case 'G':
					write_to_output(d, "Enter 'body part' skill shown to Others when the victim is found.\r\n"
							"[OLD]: %s\r\n"
							"[NEW]: ",
							((OLC_SKILL(d)->others_body_found)?OLC_SKILL(d)->others_body_found:"NULL"));

					OLC_MODE(d) = SKEDIT_VICT_OTHERS_BODY_FOUND;
					return;
				case 'h': case 'H':
					write_to_output(d, "Enter 'body part' skill shown to the Victim when the victim is found.\r\n"
							"[OLD]: %s\r\n"
							"[NEW]: ",
							((OLC_SKILL(d)->vict_body_found)?OLC_SKILL(d)->vict_body_found:"NULL"));

					OLC_MODE(d) = SKEDIT_VICT_VICT_BODY_FOUND;
					return;
				case 'i': case 'I':
					write_to_output(d, "Enter 'object' skill shown to the Character when the object is found.\r\n"
							"[OLD]: %s\r\n"
							"[NEW]: ",
							((OLC_SKILL(d)->char_obj_found)?OLC_SKILL(d)->char_obj_found:"NULL"));

					OLC_MODE(d) = SKEDIT_OBJ_CHAR_FOUND;
					return;
				case 'k': case 'K':
					write_to_output(d, "Enter 'object' skill shown to the Room when the object is found.\r\n"
							"[OLD]: %s\r\n"
							"[NEW]: ",
							((OLC_SKILL(d)->others_obj_found)?OLC_SKILL(d)->others_obj_found:"NULL"));

					OLC_MODE(d) = SKEDIT_OBJ_OTHERS_FOUND;
					return;
				default:
					skedit_disp_menu(d);
					break;
			}
			return;

		case SKEDIT_ACTION_NAME:
			if (!*arg || strchr(arg,' '))
			{
				skedit_disp_menu(d);
				return;
			} 
			if (OLC_SKILL(d)->command)
				free(OLC_SKILL(d)->command);
			OLC_SKILL(d)->command = strdup(arg);

			break;

		case SKEDIT_SORT_AS:
			if (!*arg || strchr(arg,' '))
			{
				skedit_disp_menu(d);
				return;
			}
			if (OLC_SKILL(d)->sort_as)
			{
				free(OLC_SKILL(d)->sort_as);
				OLC_SKILL(d)->sort_as = strdup(arg);
			}
			break;

		case SKEDIT_MIN_CHAR_POS:
		case SKEDIT_MIN_VICT_POS:
			if (!*arg)
			{
				skedit_disp_menu(d);
				return;
			}
			i = atoi(arg);
			if ((i < POS_DEAD) && (i > POS_STANDING))  
			{
				skedit_disp_menu(d);
				return;
			} 
			if (OLC_MODE(d) == SKEDIT_MIN_CHAR_POS)
				OLC_SKILL(d)->min_char_position = i;
			else
				OLC_SKILL(d)->min_victim_position = i;
			break;

		case SKEDIT_MIN_CHAR_LEVEL:
			if (!*arg)
			{
				skedit_disp_menu(d);
				return;
			}
			i = atoi(arg);
			if ((i < 0) && (i > LVL_IMPL))  
			{
				skedit_disp_menu(d);
				return;
			}
			OLC_SKILL(d)->min_level_char = i;
			break;

		case SKEDIT_TURN:
			if (!*arg)
			{
				skedit_disp_menu(d);
				return;
			}
			i = atoi(arg);
			if ((i < 0) || (i > 5))
			{
				skedit_disp_menu(d);
				return;
			}

			OLC_SKILL(d)->turnnum = i;
			break;
		
		case SKEDIT_TURN_DAM:
			switch (*arg)
			{
			case '1':
			write_to_output(d, "1  Է: ");
			skedit_tdam(d, arg, "0");
			return;
		case '2':
			write_to_output(d, "2  Է: ");
			skedit_tdam(d, arg, "0");
			return;
		case '3':
			write_to_output(d, "3  Է: ");
			skedit_tdam(d, arg, "0");
			return;
		case '4':
			write_to_output(d, "4  Է: ");
			skedit_tdam(d, arg, "0");
			return;
		case '5':
			write_to_output(d, "5  Է: ");
			skedit_tdam(d, arg, "0");
			return;
		default:
			disp_tdam(d);
			break;
		}
			break;
		
		
		
		case SKEDIT_NOVICT_CHAR:
			if (OLC_SKILL(d)->char_no_arg)
				free(OLC_SKILL(d)->char_no_arg);
			if (*arg)
			{
				delete_doubledollar(arg);
				OLC_SKILL(d)->char_no_arg = strdup(arg);
			}
			else 
				OLC_SKILL(d)->char_no_arg = NULL;
			break;

		case SKEDIT_NOVICT_OTHERS:
			if (OLC_SKILL(d)->others_no_arg)
				free(OLC_SKILL(d)->others_no_arg);
			if (*arg)
			{
				delete_doubledollar(arg);
				OLC_SKILL(d)->others_no_arg = strdup(arg);
			}
			else
				OLC_SKILL(d)->others_no_arg = NULL;
			break;

		case SKEDIT_VICT_CHAR_FOUND:
			if (OLC_SKILL(d)->char_found)
				free(OLC_SKILL(d)->char_found);
			if (*arg)
			{
				delete_doubledollar(arg);
				OLC_SKILL(d)->char_found = strdup(arg);
			}
			else
				OLC_SKILL(d)->char_found = NULL;
			break;

		case SKEDIT_VICT_OTHERS_FOUND:
			if (OLC_SKILL(d)->others_found)
				free(OLC_SKILL(d)->others_found);
			if (*arg)
			{
				delete_doubledollar(arg);
				OLC_SKILL(d)->others_found = strdup(arg);
			}
			else
				OLC_SKILL(d)->others_found = NULL;
			break;

		case SKEDIT_VICT_VICT_FOUND:
			if (OLC_SKILL(d)->vict_found)
				free(OLC_SKILL(d)->vict_found);
			if (*arg)
			{
				delete_doubledollar(arg);
				OLC_SKILL(d)->vict_found = strdup(arg);
			}
			else
				OLC_SKILL(d)->vict_found = NULL;
			break;

		case SKEDIT_VICT_NOT_FOUND:
			if (OLC_SKILL(d)->not_found)
				free(OLC_SKILL(d)->not_found);
			if (*arg)
			{
				delete_doubledollar(arg);
				OLC_SKILL(d)->not_found = strdup(arg);
			}
			else
				OLC_SKILL(d)->not_found = NULL;
			break;

		case SKEDIT_SELF_CHAR:
			if (OLC_SKILL(d)->char_auto)
				free(OLC_SKILL(d)->char_auto);
			if (*arg)
			{
				delete_doubledollar(arg);
				OLC_SKILL(d)->char_auto = strdup(arg);
			}
			else
				OLC_SKILL(d)->char_auto = NULL;
			break;

		case SKEDIT_SELF_OTHERS:
			if (OLC_SKILL(d)->others_auto)
				free(OLC_SKILL(d)->others_auto);
			if (*arg)
			{
				delete_doubledollar(arg);
				OLC_SKILL(d)->others_auto = strdup(arg);
			}
			else
				OLC_SKILL(d)->others_auto = NULL;
			break;

		case SKEDIT_VICT_CHAR_BODY_FOUND:
			if (OLC_SKILL(d)->char_body_found)
				free(OLC_SKILL(d)->char_body_found);
			if (*arg)
			{
				delete_doubledollar(arg);
				OLC_SKILL(d)->char_body_found = strdup(arg);
			}
			else
				OLC_SKILL(d)->char_body_found = NULL;
			break;

		case SKEDIT_VICT_OTHERS_BODY_FOUND:
			if (OLC_SKILL(d)->others_body_found)
				free(OLC_SKILL(d)->others_body_found);
			if (*arg)
			{
				delete_doubledollar(arg);
				OLC_SKILL(d)->others_body_found = strdup(arg);
			}
			else
				OLC_SKILL(d)->others_body_found = NULL;
			break;

		case SKEDIT_VICT_VICT_BODY_FOUND:
			if (OLC_SKILL(d)->vict_body_found)
				free(OLC_SKILL(d)->vict_body_found);
			if (*arg)
			{
				delete_doubledollar(arg);
				OLC_SKILL(d)->vict_body_found = strdup(arg);
			}
			else
				OLC_SKILL(d)->vict_body_found = NULL;
			break;

		case SKEDIT_OBJ_CHAR_FOUND:
			if (OLC_SKILL(d)->char_obj_found)
				free(OLC_SKILL(d)->char_obj_found);
			if (*arg)
			{
				delete_doubledollar(arg);
				OLC_SKILL(d)->char_obj_found = strdup(arg);
			}
			else
				OLC_SKILL(d)->char_obj_found = NULL;
			break;

		case SKEDIT_OBJ_OTHERS_FOUND:
			if (OLC_SKILL(d)->others_obj_found)
				free(OLC_SKILL(d)->others_obj_found);
			if (*arg)
			{
				delete_doubledollar(arg);
				OLC_SKILL(d)->others_obj_found = strdup(arg);
			}
			else
				OLC_SKILL(d)->others_obj_found = NULL;
			break;

		default:
			/* we should never get here */
			break;
	}
	OLC_VAL(d) = 1;
	skedit_disp_menu(d);
}

/*void skedit_turndam(struct descriptor_data * d, char* arg, char* arg1)
{

	switch (OLC_MODE(d))
	{
*/

ACMD(do_skstat)
{
	int i, real = FALSE;
	char arg[MAX_INPUT_LENGTH];

	if (IS_NPC(ch))
		return;

	one_argument(argument, arg);

	if (!*arg)
	{
		send_to_char(ch, "  ¸ ÷?\r\n");
		return;
	}

	for (i = 0; i <= top_of_skillt; i++)
	{
		if (is_abbrev(arg, ski_mess_list[i].command))
		{
			real = TRUE;
			break;
		}
	}

	if (!real)
	{
		send_to_char(ch, "׷ ǥ ϴ.\r\n");
		return;
	}

	get_char_colors(ch);
	send_to_char(ch, 
			"n)          : %s%-15.15s%s 1)  : %s%-15.15s%s\r\n"
			"2) ڼ[ڽ]: %s%-8.8s%s        3) ڼ[]: %s%-8.8s%s\r\n"
			"4) [ڽ]: %s%-3d%s             5) Show if Invis   : %s%s%s\r\n"
			"6)  Ƚ: %s%d%s		 : %s%-6d%s\r\n"
			"7) Char    [NO ARG]: %s%s%s\r\n"
			"8) Others  [NO ARG]: %s%s%s\r\n"
			"9) Char [NOT FOUND]: %s%s%s\r\n"
			"j) Char  [ARG SELF]: %s%s%s\r\n"
			"a) Others[ARG SELF]: %s%s%s\r\n"
			"b) Char      [VICT]: %s%s%s\r\n"
			"c) Others    [VICT]: %s%s%s\r\n"
			"d) Victim    [VICT]: %s%s%s\r\n"
			"e) Char  [BODY PRT]: %s%s%s\r\n"
			"f) Others[BODY PRT]: %s%s%s\r\n"
			"g) Victim[BODY PRT]: %s%s%s\r\n"
			"h) Char       [OBJ]: %s%s%s\r\n"
			"i) Others     [OBJ]: %s%s%s\r\n",

			yel, ski_mess_list[i].command, nrm,
			yel, ski_mess_list[i].sort_as, nrm,
			cyn, position_types[ski_mess_list[i].min_char_position], nrm,
			cyn, position_types[ski_mess_list[i].min_victim_position], nrm,
			cyn, ski_mess_list[i].min_level_char, nrm,
			cyn, (ski_mess_list[i].hide ? "HIDDEN" : "NOT HIDDEN"), nrm,
			cyn, ski_mess_list[i].turnnum, nrm, 
			cyn, ski_mess_list[i].dam, nrm,
			cyn, ski_mess_list[i].char_no_arg ? ski_mess_list[i].char_no_arg : "", nrm,
			cyn, ski_mess_list[i].others_no_arg ? ski_mess_list[i].others_no_arg : "", nrm,
			cyn, ski_mess_list[i].not_found ? ski_mess_list[i].not_found : "", nrm,
			cyn, ski_mess_list[i].char_auto ? ski_mess_list[i].char_auto : "", nrm,
			cyn, ski_mess_list[i].others_auto ? ski_mess_list[i].others_auto : "", nrm,
			cyn, ski_mess_list[i].char_found ? ski_mess_list[i].char_found : "", nrm,
			cyn, ski_mess_list[i].others_found ? ski_mess_list[i].others_found : "", nrm,
			cyn, ski_mess_list[i].vict_found ? ski_mess_list[i].vict_found : "", nrm,
			cyn, ski_mess_list[i].char_body_found ? ski_mess_list[i].char_body_found : "", nrm,
			cyn, ski_mess_list[i].others_body_found ? ski_mess_list[i].others_body_found : "", nrm,
			cyn, ski_mess_list[i].vict_body_found ? ski_mess_list[i].vict_body_found : "", nrm,
			cyn, ski_mess_list[i].char_obj_found ? ski_mess_list[i].char_obj_found : "", nrm,
			cyn, ski_mess_list[i].others_obj_found ? ski_mess_list[i].others_obj_found : "", nrm);

}

int skedit_find_command(const char *txt)
{
	int cmd;

	for (cmd = 1; *complete_cmd_info[cmd].command != '\n'; cmd++)
	{
		if (!strncmp(complete_cmd_info[cmd].sort_as, txt, strlen(txt)) ||
				!strcmp(complete_cmd_info[cmd].command, txt))
			return (cmd);
	}

	return (-1);
}

