#include "conf.h"
#include "sysdep.h"

#include "structs.h"
#include "utils.h"

const char *zod_names[] =
{
	"",
	"",
	"",
	"",
	"",
	"",
	"",
	"",
	"",
	"",
	"",
	"",
	"\n"
};

const char *zod_abbrevs[] =
{
	"",
	"",
	"",
	"",
	"",
	"",
	"",
	"",
	"",
	"",
	"",
	"",
	"\n"
};

const char *pc_zod_types[] =
{
	"()",
	"()",
	"ȣ()",
	"䳢()",
	"()",
	"()",
	"()",
	"()",
	"()",
	"()",
	"()",
	"()",
	"\n"
};

const char *zod_display[NUM_ZODES] =
{
	"1) ()\r\n",
	"2) ()\r\n",
	"3) (ȣ)\r\n",
	"4) (䳢)\r\n",
	"5) ()\r\n",
	"6) ()\r\n",
	"7) ()\r\n",
	"8) ()\r\n",
	"9) ()\r\n",
	"10) ()\r\n",
	"11) ()\r\n",
	"12) ()\r\n",
};

#define Y   TRUE
#define N   FALSE

/* Original ZODIAC Breakout */
int zod_ok_gender[NUM_SEX][NUM_ZODES] =
{
	/*        , , , , , , , , , , ,  */
	/* N */ {  Y,  Y,  Y,  Y,  Y,  Y,  Y,  Y,  Y,  Y,  Y, Y },
	/* M */ {  Y,  Y,  Y,  Y,  Y,  Y,  Y,  Y,  Y,  Y,  Y, Y },
	/* F */ {  Y,  Y,  Y,  Y,  Y,  Y,  Y,  Y,  Y,  Y,  Y, Y }
};


/*
 * The code to interpret a zodiac letter (used in interpreter.c when a
 * new character is selecting a zodiac).
 */
int parse_zod(struct char_data *ch, int arg)
{
	int zod = ZOD_UNDEFINED;

	switch (arg)
	{
		case 1:  zod = ZOD_JA      ; break;
		case 2:  zod = ZOD_CHUCK        ; break;
		case 3:  zod = ZOD_IN      ; break;
		case 4:  zod = ZOD_MYO      ; break;
		case 5:  zod = ZOD_JIN   ; break;
		case 6:  zod = ZOD_SA   ; break;
		case 7:  zod = ZOD_OH   ; break;
		case 8:  zod = ZOD_MI   ; break;
		case 9:  zod = ZOD_SIN     ; break;
		case 10: zod = ZOD_YU  ; break;
		case 11: zod = ZOD_SUL      ; break;
		case 12: zod = ZOD_HAE     ; break;
		default: zod = ZOD_UNDEFINED  ; break;
	}
	if (zod >= 0 && zod < NUM_ZODES)
		if (!zod_ok_gender[(int)GET_SEX(ch)][zod])
			zod = ZOD_UNDEFINED;

	return (zod);
}

void zodiac_ability_modifiers(struct char_data *ch)
{
	switch (GET_ZOD(ch))
	{
		case ZOD_JA:
			ch->real_abils.str = 2;
			ch->real_abils.dex = 3;
			ch->real_abils.intel = 10;
			ch->real_abils.wis = 8;
			ch->real_abils.con = 8;
			ch->real_abils.cha = 5;		
			break;
		case ZOD_CHUCK:
			ch->real_abils.str = 9;
			ch->real_abils.dex = 8;
			ch->real_abils.intel = 3;
			ch->real_abils.wis = 3;
			ch->real_abils.con = 8;
			ch->real_abils.cha = 5;		
			break;
		case ZOD_IN:
			ch->real_abils.str = 10;
			ch->real_abils.dex = 7;
			ch->real_abils.intel = 2;
			ch->real_abils.wis = 2;
			ch->real_abils.con = 7;
			ch->real_abils.cha = 8;
			break;
		case ZOD_MYO:
			ch->real_abils.str = 7;
			ch->real_abils.dex = 9;
			ch->real_abils.intel = 3;
			ch->real_abils.wis = 4;
			ch->real_abils.con = 7;
			ch->real_abils.cha = 7;    
			break;
		case ZOD_JIN:
			ch->real_abils.str = 2;
			ch->real_abils.dex = 4;
			ch->real_abils.intel = 9;
			ch->real_abils.wis = 6;
			ch->real_abils.con = 5;
			ch->real_abils.cha = 10;
			break;
		case ZOD_SA:
			ch->real_abils.str = 3;
			ch->real_abils.dex = 3;
			ch->real_abils.intel = 6;
			ch->real_abils.wis = 10;
			ch->real_abils.con = 6;
			ch->real_abils.cha = 8;
			break;
		case ZOD_OH:
			ch->real_abils.str = 8;
			ch->real_abils.dex = 6;
			ch->real_abils.intel = 4;
			ch->real_abils.wis = 5;
			ch->real_abils.con = 10;
			ch->real_abils.cha = 3;
			break;
		case ZOD_MI:
			ch->real_abils.str = 3;
			ch->real_abils.dex = 7;
			ch->real_abils.intel = 6;
			ch->real_abils.wis = 6;
			ch->real_abils.con = 9;
			ch->real_abils.cha = 5;
			break;
		case ZOD_SIN:
			ch->real_abils.str = 6;
			ch->real_abils.dex = 9;
			ch->real_abils.intel = 6;
			ch->real_abils.wis = 3;
			ch->real_abils.con = 8;
			ch->real_abils.cha = 4;
			break;
		case ZOD_YU:
			ch->real_abils.str = 4;
			ch->real_abils.dex = 8;
			ch->real_abils.intel = 6;
			ch->real_abils.wis = 6;
			ch->real_abils.con = 8;
			ch->real_abils.cha = 4;
			break;
		case ZOD_SUL:
			ch->real_abils.str = 8;
			ch->real_abils.dex = 10;
			ch->real_abils.intel = 4;
			ch->real_abils.wis = 3;
			ch->real_abils.con = 6;
			ch->real_abils.cha = 5;
			break;
		case ZOD_HAE:
			ch->real_abils.str = 8;
			ch->real_abils.dex = 8;
			ch->real_abils.intel = 4;
			ch->real_abils.wis = 4;
			ch->real_abils.con = 9;
			ch->real_abils.cha = 3;
			break;
		default:
			break;
	}

	if (GET_SEX(ch) == 1)
	{
		ch->real_abils.str += 1;
		ch->real_abils.dex -= 1;
	}
	else if (GET_SEX(ch) == 2)
	{
		ch->real_abils.str -= 1;
		ch->real_abils.dex += 1;
	}
}

/*
   void set_height_by_zod(struct char_data *ch)
   {
   if (GET_SEX(ch) == SEX_MALE)
   {
   if (IS_IN(ch))
   GET_HEIGHT(ch) = 43 + dice(1, 10);
   else if (IS_CHUCK(ch))
   GET_HEIGHT(ch) = 55 + dice(1, 10);
   else if (IS_MYO(ch))
   GET_HEIGHT(ch) = 38 + dice(1, 6);
   else // if (IS_JA(ch))
   GET_HEIGHT(ch) = 60 + dice(2, 10);
   } else // if (IS_FEMALE(ch)) 
   {
   if (IS_IN(ch))
   GET_HEIGHT(ch) = 41 + dice(1, 10);
   else if (IS_CHUCK(ch))
   GET_HEIGHT(ch) = 50 + dice(1, 10);
   else if (IS_MYO(ch))
   GET_HEIGHT(ch) = 36 + dice(1, 6);
   else // if (IS_JA(ch))
   GET_HEIGHT(ch) = 59 + dice(2, 10);
   }
   return;
   }

   void set_weight_by_zod(struct char_data *ch)
   {
   if (GET_SEX(ch) == SEX_MALE)
   {
   if (IS_IN(ch))
   GET_WEIGHT(ch) = 130 + dice(4, 10);
   else if (IS_CHUCK(ch))
   GET_WEIGHT(ch) = 90 + dice(3, 10);
   else if (IS_MYO(ch))
   GET_WEIGHT(ch) = 72 + dice(5, 4);
   else // if (IS_JA(ch)) 
   GET_WEIGHT(ch) = 140 + dice(6, 10);
   } else // if (IS_FEMALE(ch)) 
   {
   if (IS_IN(ch))
   GET_WEIGHT(ch) = 105 + dice(4, 10);
   else if (IS_CHUCK(ch))
   GET_WEIGHT(ch) = 70 + dice(3, 10);
   else if (IS_MYO(ch))
   GET_WEIGHT(ch) = 68 + dice(5, 4);
   else / if (IS_JA(ch)) /
   GET_WEIGHT(ch) = 100 + dice(6, 10);
   }
   return;
   }
   */

int invalid_zod(struct char_data *ch, struct obj_data *obj)
{
	if (GET_RACE(ch) >= LVL_IMMORT)
		return FALSE;

	if (OBJ_FLAGGED(obj, ITEM_ANTI_JA) && IS_JA(ch))
		return (TRUE);

	if (OBJ_FLAGGED(obj, ITEM_ANTI_CHUCK) && IS_CHUCK(ch))
		return (TRUE);

	if (OBJ_FLAGGED(obj, ITEM_ANTI_IN) && IS_IN(ch))
		return (TRUE);

	if (OBJ_FLAGGED(obj, ITEM_ANTI_MYO) && IS_MYO(ch))
		return (TRUE);

	if (OBJ_FLAGGED(obj, ITEM_ANTI_JIN) && IS_JIN(ch))
		return (TRUE);

	if (OBJ_FLAGGED(obj, ITEM_ANTI_SA) && IS_SA(ch))
		return (TRUE);

	if (OBJ_FLAGGED(obj, ITEM_ANTI_OH) && IS_OH(ch))
		return (TRUE);

	if (OBJ_FLAGGED(obj, ITEM_ANTI_MI) && IS_MI(ch))
		return (TRUE);

	if (OBJ_FLAGGED(obj, ITEM_ANTI_SIN) && IS_SIN(ch))
		return (TRUE);

	if (OBJ_FLAGGED(obj, ITEM_ANTI_YU) && IS_YU(ch))
		return (TRUE);

	if (OBJ_FLAGGED(obj, ITEM_ANTI_SUL) && IS_SUL(ch))
		return (TRUE);

	if (OBJ_FLAGGED(obj, ITEM_ANTI_HAE) && IS_HAE(ch))
		return (TRUE);

	return (FALSE);
}

