/* *************************************************************************
 *  File: Utils.c															*
 *  Usage: Convert a DangunMud mob file into a series of circle's .mob files*
 *                                 Copyleft Registered by savvy  2004.11.9  *
 *************************************************************************  */
#include "conf.h"
#include "sysdep.h"
#include "structs.h"
#include "dangun.h"
#include "hangul.h"

/**************************************************************************
 * declarations of local function 																				 *
 ************************************************************************ */

int get_line(FILE *fl, char *buf)
{
	char temp[READ_SIZE];
	int lines = 0;
	int sl;

	do
	{
		if (!fgets(temp, READ_SIZE, fl))
			return (0);
		lines++;
	} while (*temp == '*' || *temp == '\n' || *temp == '\r');

	/* Last line of file doesn't always have a \n, but it should. */
	sl = strlen(temp);

	while (sl > 0 && (temp[sl - 1] == '\n' || temp[sl - 1] == '\r'))
		temp[--sl] = '\0';

	strcpy(buf, temp); /* strcpy: OK, if buf >= READ_SIZE (256) */
	return (lines);
}

#if !defined(HAVE_STRDUP)
/* Create a duplicate of a string */
char * strdup(const char * source)
{
	char *new_z;

	CREATE(new_z, char, strlen(source) + 1);
	return (strcpy(new_z, source)); /* strcpy: OK */
}
#endif

/*
 * Strips \r\n from end of string.
 */
void prune_crlf(char *txt)
{
	int i = strlen(txt) - 1;

	while (txt[i] == '\n' || txt[i] == '\r')
		txt[i--] = '\0';
}


char * fread_str(FILE *fl, const char *error, bool add_new_line)
{
	char buf[MAX_STRING_LENGTH], tmp[512+1];
	char *point;
	int done = 0, length = 0, templength;
	int lines = 0;

	*buf = '\0';

	do 
	{
		if (!fgets(tmp, 512, fl)) 
		{
			printf("SYSERR: fread_string: format error at or near %s\n", error);
			exit(1);
		}

		lines++;

		if (*tmp == '%')
		{
			done = 1;
		}
		else
		{
			/* If there is a '%', end the string; else put an "\r\n" over the '\n'. */
			/* now only removes trailing ~'s -- Welcor */
			point = strchr(tmp, '\0');
			for (point--; (isspace(*point) || *point == '\r' || *point == '\n'); point--);

			if (*point == '%')
			{
				if (add_new_line || lines != 1)
					*(point++) = '\n';

				*(point) = '\0';
				done = 1;
			}
			else
			{
				//*(++point) = '\r';
				*(++point) = '\n';
				*(++point) = '\0';
			}
			
			templength = point - tmp;

			if (length + templength >= MAX_STRING_LENGTH)
			{
				printf("SYSERR: fread_string: string too large");
				printf("%s\n", error);
				exit(1);
			}
			else
			{
				strcat(buf + length, tmp);	/* strcat: OK (size checked above) */
				length += templength;
			}
		}
	} while (!done);

	/* allocate space for the new string and copy it */
	return (strlen(buf) ? strdup(buf) : NULL);
}

/* creates a random number in interval [from;to] */
int rand_number(int from, int to)
{
	/* error checking in case people call this incorrectly */
	if (from > to)
	{
		int tmp = from;
		from = to;
		to = tmp;
		printf("SYSERR: rand_number() should be called with lowest, then highest. (%d, %d), not (%d, %d).", from, to, to, from);
	}

	/*
	 * This should always be of the form:
	 *
	 *	((float)(to - from + 1) * rand() / (float)(RAND_MAX + from) + from);
	 *
	 * if you are using rand() due to historical non-randomness of the
	 * lower bits in older implementations.  We always use circle_random()
	 * though, which shouldn't have that problem. Mean and standard
	 * deviation of both are identical (within the realm of statistical
	 * identity) if the rand() implementation is non-broken.
	 */
	return ((rand() % (to - from + 1)) + from);
}

/* simulates dice roll */
int dice(int num, int size)
{
	int sum = 0;

	if (size <= 0 || num <= 0)
		return (0);

	while (num-- > 0)
		sum += rand_number(1, size);

	return (sum);
}

int sprintascii(char *out, bitvector_t bits)
{
	int i, j = 0;
	/* 32 bits, don't just add letters to try to get more unless your bitvector_t is also as large. */
	char *flags = "abcdefghijklmnopqrstuvwxyzABCDEF";

	for (i = 0; flags[i] != '\0'; i++)
		if (bits & (1 << i))
			out[j++] = flags[i];

	if (j == 0) /* Didn't write anything. */
		out[j++] = '0';

	/* NUL terminate the output string. */
	out[j++] = '\0';
	return j;
}

char * to_ascii(int bits)
{
	int i, j = 0;
	/* 32 bits, don't just add letters to try to get more unless your bitvector_t is also as large. */
	char *flags = "abcdefghijklmnopqrstuvwxyzABCDEF";
	char *tstr;
	CREATE(tstr, char, 32);
	
	for (i = 0; flags[i] != '\0'; i++)
		if (bits & (1 << i))
			tstr[j++] = flags[i];
		
	if (j == 0) /* Didn't write anything. */
		tstr[j++] = '0';

	/* NUL terminate the output string. */
	tstr[j++] = '\0';
	return tstr;
}

bitvector_t asciiflag_conv(char * flag)
{
	bitvector_t flags = 0;
	int is_num = TRUE;
	char *p;

	for (p = flag; *p; p++) 
	{
		if (islower(*p))
			flags |= 1 << (*p - 'a');
		else if (isupper(*p))
			flags |= 1 << (26 + (*p - 'A'));

		if (!isdigit(*p))
			is_num = FALSE;
	}

	//if (is_num) flags = atol(flag);
	return (flags);
}

void skip_spaces(char **string)
{
	for (; **string && isnhspace(**string); (*string)++);
}
