/* ************************************************************************ *
 *  File: objconv.c                                                          *
 *  Usage: Convert DangunMud's .obj files into series of circle's .obj files *
 *                                    									    *
 * short_descr->name Ʈ short_descriptor  ̸ó       *
 *************************************************************************** */
#include "conf.h"
#include "sysdep.h"
#include "structs.h"
#include "dangun.h"
#include "Utils.h"

#define WR_FLAGS(item)	((item).obj_flags.wear_flags)
#define EX_FLAGS(item)	((item).obj_flags.extra_flags)
#define PR_FLAGS(item)	((item).obj_flags.bitvector)
#define OBJ_VAL(obj, val) ((obj).obj_flags.value[(val)])

/**************************************************************************
 *  declarations of most of the 'global' variables                         *
 ************************************************************************ */
struct obj_data *wobj = NULL; /* arry of objs */
int char_cnt;

/**************************************************************************
 * declarations of local function 																				 *
 ************************************************************************ */
int load_obj(FILE *fl,obj_vnum virtual_nr,int obj_nr);
void write_obj(FILE *fp, obj_vnum mvnum, int obj_nr);
int do_convert(FILE * dan_f, FILE * newdan_f, char *filename);

int load_obj(FILE *fl,obj_vnum virtual_nr,int obj_nr)
{
	const char *err;
	int i, retval,lines, type_flag;
	int d_type,d_weight,d_cost,d_life;  // 1 line
	int d_class;
	int d_hit,d_dex; // 2 line
	ull_int d_wear;// 2 line
	int d_tmp;// 2 line
	ull_int d_effect;  //3 line
	int d_level;    //3 line

	char *p_alias,*p_name ;
	char *p_sdesc,*p_ldesc;
	char f_str[256],d_str[256];
	char flag_1[64],flag_2[64],flag_3[64];
	char abuf[64],bbuf[64];
	int d_char[32],d_cvalue[32];

#if defined(__DEBUG)
	printf("Start of load_obj(%d)---------------------------------\n",virtual_nr);
#endif
	wobj[obj_nr].item_number = virtual_nr;
	CREATE(p_alias,char,256);
	p_alias=(char *) fread_str(fl,err);
#if defined(__DEBUG)
	printf("Alias:\n%s\n",p_alias);
#endif 

	CREATE(p_name,char,64);
	p_name=(char *)fread_str(fl,err);
#if defined(__DEBUG)
	printf("Name:\n%s\n",p_name);
#endif 

	CREATE(wobj[obj_nr].name,char,256);
	if (p_alias)
	{
		strcpy(wobj[obj_nr].name,p_alias);
		wobj[obj_nr].name[strlen(p_alias)]='\0';
	}
	free(p_alias);

	CREATE(p_sdesc,char,MAX_STRING_LENGTH);
	CREATE(wobj[obj_nr].short_description,char,512);
	p_sdesc=(char *)fread_str(fl,err);
	if (p_name)
	{
		strcpy(wobj[obj_nr].short_description,p_name);
		wobj[obj_nr].short_description[strlen(p_name)]='\0';
	}
#if defined(__DEBUG)
	printf("SDesc:\n%s\n",wobj[obj_nr].short_description);
#endif 
	free(p_name);

	CREATE(p_ldesc,char,MAX_STRING_LENGTH);
	CREATE(wobj[obj_nr].description,char,MAX_STRING_LENGTH);
	p_ldesc=(char *)fread_str(fl,err);
	if (p_sdesc)
	{
		strcpy(wobj[obj_nr].description,p_sdesc);
		wobj[obj_nr].description[strlen(p_sdesc)]='\0';
	}
#if defined(__DEBUG)
	printf("LDesc:\n%s\n",wobj[obj_nr].description);
#endif 
	free(p_sdesc);

	CREATE(wobj[obj_nr].action_description,char,MAX_STRING_LENGTH);
	if (p_ldesc)
	{
		strcpy(wobj[obj_nr].action_description,p_ldesc);
		wobj[obj_nr].action_description[strlen(p_ldesc)]='\0';
	}
	free(p_ldesc);


	// charater's ability(action,affection...)

	lines = get_line(fl,flag_1);
#if defined(__DEBUG)
	printf("lines(%d):%s\n",lines,flag_1);
#endif  
	retval = sscanf(flag_1, "%d %d %d %d\n",&d_type,&d_weight,&d_cost,&d_life);

	lines = get_line(fl,flag_2);
#if defined(__DEBUG)
	printf("lines(%d):%s\n",lines,flag_2);
#endif 
	retval = sscanf(flag_2, "%d %d %d %d %d\n",&d_class,&d_hit,&d_dex,&d_wear,&d_tmp,&d_effect);

	lines = get_line(fl,flag_3);
#if defined(__DEBUG)
	printf("lines(%d):%s\n",lines,flag_3);
#endif 
	retval = sscanf(flag_3, "%d %d ",&d_effect,&d_level);

	switch (d_type)
	{
		case DI_TYPE_TRASH : 
			wobj[obj_nr].obj_flags.type_flag = ITEM_TRASH;
			break;
		case DI_TYPE_WEAPON : 
			wobj[obj_nr].obj_flags.type_flag = ITEM_WEAPON;
			wobj[obj_nr].obj_flags.value[1] = d_dex; // ֻ
			wobj[obj_nr].obj_flags.value[2] = d_hit; // ũ 
			wobj[obj_nr].obj_flags.value[3] = 0; //msg : hit/hits
			break;
		case DI_TYPE_ARMOR : 
			wobj[obj_nr].obj_flags.type_flag = ITEM_ARMOR;
			wobj[obj_nr].obj_flags.value[0] = d_hit; // 
			break;
		case DI_TYPE_FOOD : 
			wobj[obj_nr].obj_flags.type_flag = ITEM_FOOD;
			wobj[obj_nr].obj_flags.value[0] = 100; // 100 tick (θӽð)
			break;
		case DI_TYPE_CONTAINER : 
			wobj[obj_nr].obj_flags.type_flag = ITEM_CONTAINER;
			wobj[obj_nr].obj_flags.value[0] = d_weight*10; //capacity
			wobj[obj_nr].obj_flags.value[1] = 1; //closeable
			wobj[obj_nr].obj_flags.value[2] = -1; // key #
			wobj[obj_nr].obj_flags.value[3] = 0;
			break;
		case DI_TYPE_LIGHT : 
			wobj[obj_nr].obj_flags.type_flag = ITEM_LIGHT;
			wobj[obj_nr].obj_flags.value[2] = -1;
			break;
		case DI_TYPE_MONEY : 
			wobj[obj_nr].obj_flags.value[0] = d_cost; // ٹ
			wobj[obj_nr].obj_flags.type_flag = ITEM_MONEY;
			break;
		case DI_TYPE_NOTE : 
			wobj[obj_nr].obj_flags.type_flag = ITEM_NOTE;
			break;
		case DI_TYPE_CORPSE : 
			wobj[obj_nr].obj_flags.type_flag = ITEM_TRASH; //?
			break;
		case DI_TYPE_SCROLL : //  ʿ ( ̿)
			wobj[obj_nr].obj_flags.type_flag = ITEM_SCROLL;
			wobj[obj_nr].obj_flags.value[0] = d_level;
			wobj[obj_nr].obj_flags.value[1] = 1; //spell #
			wobj[obj_nr].obj_flags.value[2] = -1;
			wobj[obj_nr].obj_flags.value[3] = -1;
			break;
		case DI_TYPE_KEY : 
			wobj[obj_nr].obj_flags.type_flag = ITEM_KEY;
			break;
		case DI_TYPE_DRINKCON : 
			wobj[obj_nr].obj_flags.type_flag = ITEM_DRINKCON;
			wobj[obj_nr].obj_flags.value[0] = d_weight; //capacity
			wobj[obj_nr].obj_flags.value[1] = d_weight; //spell #
			wobj[obj_nr].obj_flags.value[2] = 0; //LIQ_WATER
			wobj[obj_nr].obj_flags.value[3] = 0; //Not Poisoned
			break;
		case DI_TYPE_TREASURE : 
			wobj[obj_nr].obj_flags.type_flag = ITEM_TREASURE;
			break;
		case DI_TYPE_WAND : // ܶ Ȱ뵵...?? 
			wobj[obj_nr].obj_flags.type_flag = ITEM_WAND;
			wobj[obj_nr].obj_flags.value[0] = d_level;
			wobj[obj_nr].obj_flags.value[1] = 1;
			wobj[obj_nr].obj_flags.value[2] = 1;
			wobj[obj_nr].obj_flags.value[3] = 1; //spell #
			break;
		case DI_TYPE_MISSILE : 
			wobj[obj_nr].obj_flags.type_flag = ITEM_WAND; // not implemented
			break;
		case DI_TYPE_TRANSFER : 
			wobj[obj_nr].obj_flags.type_flag = ITEM_VEHICLE; // ...??
			break;
		default :
			printf("Undefined type(%d) founded!\n",d_type); //ITEM Ÿ ã ....
			getchar();
	}

	type_flag = wobj[obj_nr].obj_flags.type_flag;

	if (type_flag == ITEM_WEAPON)
	{
		if (IS_SET(d_class, DI_CLASS_2H))
			SET_BIT_AR(EX_FLAGS(wobj[obj_nr]), ITEM_2H);

		switch (d_class)
		{
			/*
			case DI_CLASS_ETC:
				wobj[obj_nr].obj_flags.type_flag = ITEM_WEAPON; //Ǽ/
				wobj[obj_nr].obj_flags.value[1] = d_dex; // ֻ
				wobj[obj_nr].obj_flags.value[2] = d_hit; // ũ 
				wobj[obj_nr].obj_flags.value[3] = 0; //msg : hit/hits
				break;
			*/
			case DI_CLASS_SWORD:
			case DI_CLASS_SWORD+DI_CLASS_2H:
			case 196609: // 
				wobj[obj_nr].obj_flags.type_flag = ITEM_WEAPON; //
				wobj[obj_nr].obj_flags.value[1] = d_dex; // ֻ
				wobj[obj_nr].obj_flags.value[2] = d_hit; // ũ 
				wobj[obj_nr].obj_flags.value[3] = 3; //msg : slash/slashes
				break;

			case DI_CLASS_SPEAR:
			case DI_CLASS_SPEAR+DI_CLASS_2H:
				wobj[obj_nr].obj_flags.type_flag = ITEM_WEAPON; //â
				wobj[obj_nr].obj_flags.value[1] = d_dex; // ֻ
				wobj[obj_nr].obj_flags.value[2] = d_hit; // ũ 
				wobj[obj_nr].obj_flags.value[3] = 11; //msg : pierce/pierces
				break;

			case DI_CLASS_STAFF+DI_CLASS_2H:
				wobj[obj_nr].obj_flags.type_flag = ITEM_STAFF; //
				wobj[obj_nr].obj_flags.value[0] = d_level;
				wobj[obj_nr].obj_flags.value[1] = 1;
				wobj[obj_nr].obj_flags.value[2] = 1;
				wobj[obj_nr].obj_flags.value[3] = 1; //spell #
				break;

			case DI_CLASS_HEAVY:
			case DI_CLASS_HEAVY+DI_CLASS_2H:
				wobj[obj_nr].obj_flags.type_flag = ITEM_WEAPON; //ߺ
				wobj[obj_nr].obj_flags.value[1] = d_dex; // ֻ
				wobj[obj_nr].obj_flags.value[2] = d_hit; // ũ 
				wobj[obj_nr].obj_flags.value[3] = 7; //msg : pound/pounds
				break;

			case DI_CLASS_FIST:
				wobj[obj_nr].obj_flags.type_flag = ITEM_WEAPON; //ָ
				wobj[obj_nr].obj_flags.value[1] = d_dex; // ֻ
				wobj[obj_nr].obj_flags.value[2] = d_hit; // ũ 
				wobj[obj_nr].obj_flags.value[3] = 13; //msg : punch/puches
				break;

			case DI_CLASS_KNIFE:
			case DI_CLASS_KNIFE+DI_CLASS_2H:
				wobj[obj_nr].obj_flags.type_flag = ITEM_WEAPON; //AFF bit set
				wobj[obj_nr].obj_flags.value[1] = d_dex; // ֻ
				wobj[obj_nr].obj_flags.value[2] = d_hit; // ũ 
				wobj[obj_nr].obj_flags.value[3] = 14; //msg : stab/stabs
				SET_BIT_AR(EX_FLAGS(wobj[obj_nr]),ITEM_2H);	
				break;

			case DI_CLASS_MACE:
			case DI_CLASS_MACE+DI_CLASS_2H:
				wobj[obj_nr].obj_flags.type_flag = ITEM_WEAPON; //
				wobj[obj_nr].obj_flags.value[1] = d_dex; // ֻ
				wobj[obj_nr].obj_flags.value[2] = d_hit; // ũ 
				wobj[obj_nr].obj_flags.value[3] = 6; //msg : crush
				break;

			case DI_CLASS_PAN:
				wobj[obj_nr].obj_flags.type_flag = ITEM_WEAPON; //(ä)
				wobj[obj_nr].obj_flags.value[1] = d_dex; // ֻ
				wobj[obj_nr].obj_flags.value[2] = d_hit; // ũ 
				wobj[obj_nr].obj_flags.value[3] = 12; //msg : blast/blasts
				break;

			case DI_CLASS_ROD:
				wobj[obj_nr].obj_flags.type_flag = ITEM_WEAPON; //(ä)
				wobj[obj_nr].obj_flags.value[1] = d_dex; // ֻ
				wobj[obj_nr].obj_flags.value[2] = d_hit; // ũ 
				wobj[obj_nr].obj_flags.value[3] = 2; //msg : whip/whips
				break;

			case DI_CLASS_BOW:
				wobj[obj_nr].obj_flags.type_flag = ITEM_WEAPON; //Ȱ
				wobj[obj_nr].obj_flags.value[1] = d_dex; // ֻ
				wobj[obj_nr].obj_flags.value[2] = d_hit; // ũ 
				wobj[obj_nr].obj_flags.value[3] = 1; //msg : sting/stings
				break;

			default :
				printf("Undefined weapon class(%d) founded!\n",d_class); //ITEM Ÿ ã ....
				getchar();
		}
	}

	if (type_flag == ITEM_DRINKCON)
	{
		switch (d_class)
		{
			case 0:
				break;

			case 1:
				wobj[obj_nr].obj_flags.type_flag = ITEM_FOUNTAIN; // container type:3
				wobj[obj_nr].obj_flags.value[0] = d_weight; //capacity
				wobj[obj_nr].obj_flags.value[1] = d_weight; //spell #
				wobj[obj_nr].obj_flags.value[2] = 0; //LIQ_WATER
				wobj[obj_nr].obj_flags.value[3] = 0; //Not Poisoned
				break;

			case 11:
				wobj[obj_nr].obj_flags.type_flag = ITEM_DRINKCON; // container type:3
				wobj[obj_nr].obj_flags.value[0] = d_weight; //capacity
				wobj[obj_nr].obj_flags.value[1] = d_weight; //spell #
				wobj[obj_nr].obj_flags.value[2] = 11; //LIQ_TEA
				wobj[obj_nr].obj_flags.value[3] = 0; //Not Poisoned
				break;

			case 12:
				wobj[obj_nr].obj_flags.type_flag = ITEM_DRINKCON; // container type:4
				wobj[obj_nr].obj_flags.value[0] = d_weight; //capacity
				wobj[obj_nr].obj_flags.value[1] = d_weight; //spell #
				wobj[obj_nr].obj_flags.value[2] = 6; //LIQ_LEMONADE
				wobj[obj_nr].obj_flags.value[3] = 0; //Not Poisoned
				break;

			default :
				printf("Undefined drink class(%d) founded!\n", d_class); // ITEM Ÿ ã ....
				break;

		}
	}

	if (type_flag == ITEM_FOOD)
	{
		switch (d_class)
		{
			case 0:
				break;

			case 1: 
				wobj[obj_nr].obj_flags.type_flag = ITEM_FOOD; //԰Ÿ food type: 
				break;

			case 4:
				wobj[obj_nr].obj_flags.type_flag = ITEM_FOOD; //԰Ÿ food type: 
				break;

			case 5:
				wobj[obj_nr].obj_flags.type_flag = ITEM_FOOD; //԰Ÿ food type: ȣڿ,
				break;

			case 7:
				wobj[obj_nr].obj_flags.type_flag = ITEM_FOOD; //԰Ÿ food type:  
				break;

			default:
				printf("Undefined food class(%d) founded!\n",d_class); //ITEM Ÿ ã ....
				break;
		}
	}

	wobj[obj_nr].obj_flags.weight = d_weight;
	wobj[obj_nr].obj_flags.cost = d_cost;
	wobj[obj_nr].obj_flags.cost_per_day = 0; 
	//wobj[obj_nr].obj_flags.timer = d_life;

	/* Proccessing obj wear flag */
	// SET_BIT_AR(MOB_FLAGS(wobj[obj_nr]),MOB_ISNPC);
	if (d_wear != 0)	//   ִ ̸,   ִ.
		SET_BIT_AR(WR_FLAGS(wobj[obj_nr]), ITEM_WEAR_TAKE);

	if (IS_SET(d_wear, DI_WEAR_HEAD)) SET_BIT_AR(WR_FLAGS(wobj[obj_nr]), ITEM_WEAR_HEAD);
	if (IS_SET(d_wear, DI_WEAR_NECK)) SET_BIT_AR(WR_FLAGS(wobj[obj_nr]), ITEM_WEAR_NECK);
	if (IS_SET(d_wear, DI_WEAR_ARMS)) SET_BIT_AR(WR_FLAGS(wobj[obj_nr]), ITEM_WEAR_ARMS);
	if (IS_SET(d_wear, DI_WEAR_BODY)) SET_BIT_AR(WR_FLAGS(wobj[obj_nr]), ITEM_WEAR_BODY);
	if (IS_SET(d_wear, DI_WEAR_WINGS)) SET_BIT_AR(WR_FLAGS(wobj[obj_nr]), ITEM_WEAR_WINGS);
	if (IS_SET(d_wear, DI_WEAR_WRIST)) SET_BIT_AR(WR_FLAGS(wobj[obj_nr]), ITEM_WEAR_WRIST);
	if (IS_SET(d_wear, DI_WEAR_HAND)) SET_BIT_AR(WR_FLAGS(wobj[obj_nr]), ITEM_WEAR_HANDS);
	if (IS_SET(d_wear, DI_WEAR_WIELD)) SET_BIT_AR(WR_FLAGS(wobj[obj_nr]), ITEM_WEAR_WIELD);
	if (IS_SET(d_wear, DI_WEAR_HOLD)) SET_BIT_AR(WR_FLAGS(wobj[obj_nr]), ITEM_WEAR_HOLD); 
	if (IS_SET(d_wear, DI_WEAR_WAIST)) SET_BIT_AR(WR_FLAGS(wobj[obj_nr]), ITEM_WEAR_WAIST);
	if (IS_SET(d_wear, DI_WEAR_LEGS)) SET_BIT_AR(WR_FLAGS(wobj[obj_nr]), ITEM_WEAR_LEGS); 
	if (IS_SET(d_wear, DI_WEAR_KNEE)) SET_BIT_AR(WR_FLAGS(wobj[obj_nr]), ITEM_WEAR_LEGS);
	if (IS_SET(d_wear, DI_WEAR_EAR)) SET_BIT_AR(WR_FLAGS(wobj[obj_nr]), ITEM_WEAR_EAR); 

	/* proccessing Extra effects */
	//if (IS_SET(d_effect, DI_EX_ANTI_NOJOB)) SET_BIT_AR(EX_FLAGS(wobj[obj_nr]), ITEM_INVISIBLE); // FIXME
	if (IS_SET(d_effect, DI_EX_ANTI_PALADIN1)) SET_BIT_AR(EX_FLAGS(wobj[obj_nr]),ITEM_ANTI_WARRIOR);
	if (IS_SET(d_effect, DI_EX_ANTI_THIEF)) SET_BIT_AR(EX_FLAGS(wobj[obj_nr]),ITEM_ANTI_THIEF);
	if (IS_SET(d_effect, DI_EX_ANTI_CLERIC)) SET_BIT_AR(EX_FLAGS(wobj[obj_nr]),ITEM_ANTI_CLERIC);
	if (IS_SET(d_effect, DI_EX_ANTI_MAGIC1)) SET_BIT_AR(EX_FLAGS(wobj[obj_nr]),ITEM_ANTI_MAGIC_USER);
	if (IS_SET(d_effect, DI_EX_ANTI_MAGIC2)) SET_BIT_AR(EX_FLAGS(wobj[obj_nr]),ITEM_ANTI_MAGIC_USER);
	if (IS_SET(d_effect, DI_EX_INVISIBLE)) SET_BIT_AR(EX_FLAGS(wobj[obj_nr]),ITEM_INVISIBLE);
	if (IS_SET(d_effect, DI_EX_NOFIX)) SET_BIT_AR(EX_FLAGS(wobj[obj_nr]),ITEM_BROKEN);	
	if (IS_SET(d_effect, DI_EX_UNBREAKABLE1)) SET_BIT_AR(EX_FLAGS(wobj[obj_nr]),ITEM_UNBREAKABLE);	
	//if (IS_SET(d_effect, DI_EX_AUTO_BREAK1)) SET_BIT_AR(EX_FLAGS(wobj[obj_nr]),ITEM_SENSE_LIFE);	
	//if (IS_SET(d_effect, DI_EX_AUTO_BREAK2)) SET_BIT_AR(EX_FLAGS(wobj[obj_nr]),ITEM_POISON);	
	if (IS_SET(d_effect, DI_EX_NO_DROP)) SET_BIT_AR(EX_FLAGS(wobj[obj_nr]),ITEM_NODROP);	
	//if (IS_SET(d_effect, DI_EX_NO_SAVE)) SET_BIT_AR(EX_FLAGS(wobj[obj_nr]),ITEM_MAGICONLY);
	//if (IS_SET(d_effect, DI_EX_NO_TAKE)) SET_BIT_AR(EX_FLAGS(wobj[obj_nr]),ITEM_HIDE);
	//if (IS_SET(d_effect, DI_EX_ANTI_MALE)) SET_BIT_AR(EX_FLAGS(wobj[obj_nr]),ITEM_HIDE);	
	//if (IS_SET(d_effect, DI_EX_ANTI_FEMALE)) SET_BIT_AR(EX_FLAGS(wobj[obj_nr]),ITEM_HIDE);	
	//if (IS_SET(d_effect, DI_EX_ANTI_NOGENDER)) SET_BIT_AR(EX_FLAGS(wobj[obj_nr]),ITEM_HIDE);
	if (IS_SET(d_effect, DI_EX_ANTI_PALADIN2)) SET_BIT_AR(EX_FLAGS(wobj[obj_nr]),ITEM_ANTI_WARRIOR);
	//if (IS_SET(d_effect, DI_EX_NO_CONTAIN)) SET_BIT_AR(EX_FLAGS(wobj[obj_nr]),ITEM_HIDE);	
	//if (IS_SET(d_effect, DI_EX_NO_PICKUP)) SET_BIT_AR(EX_FLAGS(wobj[obj_nr]),ITEM_HIDE);
	//if (IS_SET(d_effect, DI_EX_NO_RENT)) SET_BIT_AR(EX_FLAGS(wobj[obj_nr]),ITEM_HIDE);
	if (IS_SET(d_effect, DI_EX_UNBREAKABLE2)) SET_BIT_AR(EX_FLAGS(wobj[obj_nr]),ITEM_UNBREAKABLE);
	if (IS_SET(d_effect, DI_EX_NO_DEATH)) SET_BIT_AR(EX_FLAGS(wobj[obj_nr]),ITEM_UNBREAKABLE);	

	sprintascii(abuf,WR_FLAGS(wobj[obj_nr])[0]);
	sprintascii(bbuf,EX_FLAGS(wobj[obj_nr])[0]);
	sprintf(f_str,"WearFlag:(%s) ExFlag:(%s)\n",abuf,bbuf);
#if defined(__DEBUG)
	puts(f_str);
#endif
	char_cnt = 0;
	for (;;){
		lines=get_line(fl,d_str);
		d_str[strlen(d_str)]='\0';
#if defined(__DEBUG)
		printf("lines(%d):%s\n",lines,d_str);
#endif
		retval = sscanf(d_str,"%d %d\n", d_char+char_cnt, d_cvalue+char_cnt); 

		//if (*d_str=='C') continue;
		if (*d_str=='E')
		{
#if defined(__DEBUG)
			printf("End of load_obj(%d)---------------------------------\n",virtual_nr);
#endif
			return 0;
		}

		if (char_cnt > 5) continue;

		switch (d_char[char_cnt])
		{
			case  DI_APPLY_NON :	//
				wobj[obj_nr].affected[char_cnt].location = d_char[char_cnt];
				wobj[obj_nr].affected[char_cnt].modifier = d_cvalue[char_cnt];
				break;
			case  DI_APPLY_STR :	//
				wobj[obj_nr].affected[char_cnt].location = d_char[char_cnt];
				wobj[obj_nr].affected[char_cnt].modifier = d_cvalue[char_cnt];
				break;
			case  DI_APPLY_DEX :	//
				wobj[obj_nr].affected[char_cnt].location = d_char[char_cnt];
				wobj[obj_nr].affected[char_cnt].modifier = d_cvalue[char_cnt];
				break;
			case  DI_APPLY_INT :	//
				wobj[obj_nr].affected[char_cnt].location = d_char[char_cnt];
				wobj[obj_nr].affected[char_cnt].modifier = d_cvalue[char_cnt];
				break;
			case  DI_APPLY_WIS :	//
				wobj[obj_nr].affected[char_cnt].location = d_char[char_cnt];
				wobj[obj_nr].affected[char_cnt].modifier = d_cvalue[char_cnt];
				break;
			case  DI_APPLY_CON :	//
				wobj[obj_nr].affected[char_cnt].location = d_char[char_cnt];
				wobj[obj_nr].affected[char_cnt].modifier = d_cvalue[char_cnt];
				break;
			case  DI_APPLY_CHA :	//
				wobj[obj_nr].affected[char_cnt].location = d_char[char_cnt];
				wobj[obj_nr].affected[char_cnt].modifier = d_cvalue[char_cnt];
				break;
			case  DI_APPLY_LCK :	// Ŭ  x
				wobj[obj_nr].affected[char_cnt].location = APPLY_SAVING_SPELL;
				wobj[obj_nr].affected[char_cnt].modifier = d_cvalue[char_cnt];
				break;
			case  DI_APPLY_INVIS :	// x
				wobj[obj_nr].affected[char_cnt].location = APPLY_SAVING_SPELL;
				wobj[obj_nr].affected[char_cnt].modifier = d_cvalue[char_cnt];
				break;
			case  DI_APPLY_SEE_INVIS :	// x
				wobj[obj_nr].affected[char_cnt].location = APPLY_SAVING_SPELL;
				wobj[obj_nr].affected[char_cnt].modifier = d_cvalue[char_cnt];
				break;
			case  DI_APPLY_HIT :	//ְü
				wobj[obj_nr].affected[char_cnt].location = APPLY_HIT ;
				wobj[obj_nr].affected[char_cnt].modifier = d_cvalue[char_cnt];
				break;
			case  DI_APPLY_MANA :	//
				wobj[obj_nr].affected[char_cnt].location = APPLY_MANA ;
				wobj[obj_nr].affected[char_cnt].modifier = d_cvalue[char_cnt];
				break;
			case  DI_APPLY_MOVE :	//̵
				wobj[obj_nr].affected[char_cnt].location = APPLY_MOVE ;
				wobj[obj_nr].affected[char_cnt].modifier = d_cvalue[char_cnt];
				break;
			case  DI_APPLY_AC :	//
				wobj[obj_nr].affected[char_cnt].location = APPLY_AC ;
				wobj[obj_nr].affected[char_cnt].modifier = d_cvalue[char_cnt];
				break;
			case  DI_APPLY_THAC0 :	//ȸǷ --> x
				wobj[obj_nr].affected[char_cnt].location = APPLY_AC;
				wobj[obj_nr].affected[char_cnt].modifier = d_cvalue[char_cnt];
				break;
			case  DI_APPLY_DAMROLL :	//ı
				wobj[obj_nr].affected[char_cnt].location = APPLY_DAMROLL;
				wobj[obj_nr].affected[char_cnt].modifier = d_cvalue[char_cnt];
				break;
			case  DI_APPLY_HITROLL :	//Ȯ
				wobj[obj_nr].affected[char_cnt].location = APPLY_HITROLL;
				wobj[obj_nr].affected[char_cnt].modifier = d_cvalue[char_cnt];
				break;
			case  DI_APPLY_REFRESH :	//ȸ x
				wobj[obj_nr].affected[char_cnt].location = APPLY_AC;
				wobj[obj_nr].affected[char_cnt].modifier = d_cvalue[char_cnt];
				break;
			case  DI_APPLY_NOENEMY :	// x
				wobj[obj_nr].affected[char_cnt].location = APPLY_DAMROLL;
				wobj[obj_nr].affected[char_cnt].modifier = d_cvalue[char_cnt];
				break;
			case  DI_APPLY_SAV_POISON :	// x
				wobj[obj_nr].affected[char_cnt].location = APPLY_SAVING_SPELL;
				wobj[obj_nr].affected[char_cnt].modifier = d_cvalue[char_cnt];
				break;
			case  DI_APPLY_PROTECT :	//ȭ
				wobj[obj_nr].affected[char_cnt].location = APPLY_AC;
				wobj[obj_nr].affected[char_cnt].modifier = d_cvalue[char_cnt];
				break;
			case  DI_APPLY_SAV_PARAL :	// x
				wobj[obj_nr].affected[char_cnt].location = APPLY_SAVING_PARA;
				wobj[obj_nr].affected[char_cnt].modifier = d_cvalue[char_cnt];
				break;
			case  DI_APPLY_SWIM :	// x
				wobj[obj_nr].affected[char_cnt].location = APPLY_SAVING_SPELL; 
				wobj[obj_nr].affected[char_cnt].modifier = d_cvalue[char_cnt];
				break;
			case  DI_APPLY_SAV_SPELL2 :	// x
				wobj[obj_nr].affected[char_cnt].location = APPLY_SAVING_SPELL;
				wobj[obj_nr].affected[char_cnt].modifier = d_cvalue[char_cnt];
				break;
			case  DI_APPLY_SAV_SPELL3 :	// x
				wobj[obj_nr].affected[char_cnt].location = APPLY_SAVING_SPELL;
				wobj[obj_nr].affected[char_cnt].modifier = d_cvalue[char_cnt];
				break;
			case  DI_APPLY_SAV_SPELL4 :	//ȥ x
				wobj[obj_nr].affected[char_cnt].location = APPLY_SAVING_SPELL;
				wobj[obj_nr].affected[char_cnt].modifier = d_cvalue[char_cnt];
				break;
			case  DI_APPLY_SAV_SPELL5 :	//ݹݻx
				wobj[obj_nr].affected[char_cnt].location = APPLY_SAVING_SPELL;
				wobj[obj_nr].affected[char_cnt].modifier = d_cvalue[char_cnt];
				break;
			case  DI_APPLY_SAV_SPELL6 :	//x
				wobj[obj_nr].affected[char_cnt].location = APPLY_SAVING_SPELL;
				wobj[obj_nr].affected[char_cnt].modifier = d_cvalue[char_cnt];
				break;
			case  DI_APPLY_SAV_SPELL7 :	//x
				wobj[obj_nr].affected[char_cnt].location = APPLY_SAVING_SPELL;
				wobj[obj_nr].affected[char_cnt].modifier = d_cvalue[char_cnt];
				break;
			case  DI_APPLY_SAV_SPELL8 :	//x
				wobj[obj_nr].affected[char_cnt].location = APPLY_SAVING_SPELL;
				wobj[obj_nr].affected[char_cnt].modifier = d_cvalue[char_cnt];
				break;
			case  DI_APPLY_SAV_SPELL9 :	//x
				wobj[obj_nr].affected[char_cnt].location = APPLY_SAVING_SPELL;
				wobj[obj_nr].affected[char_cnt].modifier = d_cvalue[char_cnt];
				break;
			case  DI_APPLY_SAV_SPELL10 :	//x
				wobj[obj_nr].affected[char_cnt].location = APPLY_SAVING_SPELL;
				wobj[obj_nr].affected[char_cnt].modifier = d_cvalue[char_cnt];
				break;
			case  DI_APPLY_SAV_SPELL11 :	//x
				wobj[obj_nr].affected[char_cnt].location = APPLY_SAVING_SPELL;
				wobj[obj_nr].affected[char_cnt].modifier = d_cvalue[char_cnt];
				break;
			default :
				printf("Undefined characteristics(%d) founded!\n",d_char[char_cnt]); //Ư  ã ....
				getchar();
		}

		char_cnt++;

	}
}

void write_obj(FILE *fp, obj_vnum ovnum, int obj_nr)
{
	struct obj_data *obj;
	struct extra_descr_data *ex_desc;
	int counter;

	char buf[MAX_STRING_LENGTH];
	char ldesc[MAX_STRING_LENGTH], sdesc[MAX_STRING_LENGTH];
	char ebuf1[MAX_STRING_LENGTH], ebuf2[MAX_STRING_LENGTH];
	char ebuf3[MAX_STRING_LENGTH], ebuf4[MAX_STRING_LENGTH];
	char wbuf1[MAX_STRING_LENGTH], wbuf2[MAX_STRING_LENGTH];
	char wbuf3[MAX_STRING_LENGTH], wbuf4[MAX_STRING_LENGTH];
	char pbuf1[MAX_STRING_LENGTH], pbuf2[MAX_STRING_LENGTH];
	char pbuf3[MAX_STRING_LENGTH], pbuf4[MAX_STRING_LENGTH];

	CREATE(obj,struct obj_data,1);

	*obj=wobj[obj_nr];
	/*
	   if (wobj[obj_nr].name!=NULL)
	   {
	   memcpy(obj->name, wobj[obj_nr].name,strlen(wobj[obj_nr].name));
	   }
	   if (wobj[obj_nr].short_description!=NULL)
	   strncpy(obj->short_description, wobj[obj_nr].short_description,strlen(wobj[obj_nr].short_description));

	   if (wobj[obj_nr].description!=NULL){
	   strncpy(obj->description,wobj[obj_nr].description,strlen(wobj[obj_nr].description));
	   }
	   */

	fprintf(fp,
			"#%d\n"
			"%s~\n"
			"%s~\n"
			"%s~\n"
			"%s~\n",
			ovnum,
			wobj[obj_nr].name,
			wobj[obj_nr].short_description,
			wobj[obj_nr].description,
			""
		   );
	sprintascii(ebuf1, EX_FLAGS(wobj[obj_nr])[0]);
	ebuf1[strlen(ebuf1)]='\0';
	sprintascii(ebuf2, EX_FLAGS(wobj[obj_nr])[1]);
	ebuf2[strlen(ebuf2)]='\0';
	sprintascii(ebuf3, EX_FLAGS(wobj[obj_nr])[2]);
	ebuf3[strlen(ebuf3)]='\0';
	sprintascii(ebuf4, EX_FLAGS(wobj[obj_nr])[3]);
	ebuf4[strlen(ebuf4)]='\0';
	sprintascii(wbuf1, WR_FLAGS(wobj[obj_nr])[0]);
	wbuf1[strlen(wbuf1)]='\0';
	sprintascii(wbuf2, WR_FLAGS(wobj[obj_nr])[1]);
	wbuf1[strlen(wbuf2)]='\0';
	sprintascii(wbuf3, WR_FLAGS(wobj[obj_nr])[2]);
	wbuf1[strlen(wbuf3)]='\0';
	sprintascii(wbuf4, WR_FLAGS(wobj[obj_nr])[3]);
	wbuf1[strlen(wbuf4)]='\0';
	sprintascii(pbuf1, PR_FLAGS(wobj[obj_nr])[0]);
	pbuf1[strlen(pbuf1)]='\0';
	sprintascii(pbuf2, PR_FLAGS(wobj[obj_nr])[1]);
	pbuf1[strlen(pbuf2)]='\0';
	sprintascii(pbuf3, PR_FLAGS(wobj[obj_nr])[2]);
	pbuf1[strlen(pbuf3)]='\0';
	sprintascii(pbuf4, PR_FLAGS(wobj[obj_nr])[3]);
	pbuf1[strlen(pbuf4)]='\0';

	fprintf(fp,
			"%d %s %s %s %s %s %s %s %s %s %s %s %s\n"
			"%d %d %d %d %d %d %d %d\n"
			"%d %d %d %d\n",

			wobj[obj_nr].obj_flags.type_flag, 
			ebuf1, ebuf2, ebuf3, ebuf4,
			wbuf1, wbuf2, wbuf3, wbuf4,
			pbuf1, pbuf2, pbuf3, pbuf4,
			OBJ_VAL(wobj[obj_nr], 0), OBJ_VAL(wobj[obj_nr], 1), OBJ_VAL(wobj[obj_nr], 2), 
			OBJ_VAL(wobj[obj_nr], 3), OBJ_VAL(wobj[obj_nr], 4), OBJ_VAL(wobj[obj_nr], 5),
			OBJ_VAL(wobj[obj_nr], 6), OBJ_VAL(wobj[obj_nr], 7),
			wobj[obj_nr].obj_flags.weight, wobj[obj_nr].obj_flags.cost, \
			wobj[obj_nr].obj_flags.cost_per_day, wobj[obj_nr].obj_flags.level
		   );

	if (wobj[obj_nr].ex_description)
	{	/* Yes, save them too. */
		for (ex_desc = wobj[obj_nr].ex_description; ex_desc; ex_desc = ex_desc->next)
		{
			/*
			 * Sanity check to prevent nasty protection faults.
			 */
			if (!ex_desc->keyword || !ex_desc->description || !*ex_desc->keyword || !*ex_desc->description)
			{
				printf("SYSERR: Corrupt ex_desc!");
				continue;
			}
			strncpy(buf, ex_desc->description, sizeof(buf) - 1);
			//strip_cr(buf);
			fprintf(fp, "E\n"
					"%s~\n"
					"%s~\n", ex_desc->keyword, buf);
		}
	}
	/*
	 * Do we have affects? 
	 */
	for (counter = 0; counter < MAX_OBJ_AFFECT; counter++) 
		if (wobj[obj_nr].affected[counter].modifier!=0)
			fprintf(fp, "A\n"
					"%d %d\n", wobj[obj_nr].affected[counter].location,
					wobj[obj_nr].affected[counter].modifier);

}


int do_convert(FILE * dan_f, FILE * newdan_f, char *filename)
{
	const char *err ;
	char mhead[128];
	int virtual_nr, obj_nr, counter,lines,i;


	CREATE(wobj,struct obj_data,256);

	obj_nr=0;

#if defined(__DEBUG)
	printf(" Start of obj=========================================(obj:%d)\n",obj_nr);
#endif
	for (;;)
	{
		lines=get_line(dan_f,mhead);	
		mhead[strlen(mhead)]='\0';
#if defined(__DEBUG)
		printf("lines(%d):%s\n",lines,mhead);
#endif
		if (mhead[0] == '+')
		{		/* start obj file */
			sscanf(mhead, "+%d\n", &virtual_nr);

			load_obj(dan_f,virtual_nr,obj_nr);
			write_obj(newdan_f,virtual_nr,obj_nr);

			if (wobj[obj_nr].name) free(wobj[obj_nr].name);
			if (wobj[obj_nr].short_description) free(wobj[obj_nr].short_description);
			if (wobj[obj_nr].description) free(wobj[obj_nr].description);
			//if (wobj[obj_nr].action_description) free(wobj[obj_nr].action_description);
			//if (wobj[obj_nr].ex_description) free(wobj[obj_nr].ex_description);
			//free(&wobj[obj_nr]);

			obj_nr++;

		}
		else
		{
			if (mhead[0] == '@')
			{	/* EOF */
				fprintf(newdan_f, "$~\n");
				break;
			}
			else
			{
				printf("%s: Wrong format detected, conversion aborted!\n", filename);
				free(wobj);
				return (1);
			}
		}
	}
#if defined(__DEBUG)
	printf(" End of obj===========================================(obj:%d)\n",obj_nr);
#endif
	free(wobj);
	return (0);
}

int main(int argc, char *argv[])
{
	FILE *idxfp,*dfp, *ndfp;
	char fn[256], part[256], nfn[256];
	char idx_f[256],e_msg[256];
	int result, index, lines,obj_nr,retval,idx_flag;

	if (argc < 2)
	{
		printf("Usage: objconv [Index] [<infile.mul> <outfile.obj>]\n");
		exit(1);
	}
	if (!strcmp(argv[1],"Index"))
	{
		printf("Converting each file \n");
		strcpy(fn,argv[1]);
		idx_flag=1;
	}
	else strcpy(fn,"Index");

	if (idx_flag==1){
		idxfp = fopen("Index","r");
		if (idxfp == NULL)
		{
			strcat(e_msg, " Index: could not be opened");
			perror(e_msg);
			return(1);
		}
		else
		{
			for (;;)
			{
				lines=get_line(idxfp,idx_f);
				idx_f[strlen(idx_f)]='\0';
#if defined(__DEBUG)
				printf("lines(%d):%s\n",lines,idx_f);
#endif
				if (idx_f[0]=='$')
				{
					break;
				}

				retval=sscanf(idx_f,"%s %d\n",fn,&obj_nr);

				sprintf(nfn,"%d.obj",obj_nr);

#if defined(__DEBUG)
				printf("fn/obj_nr:%s/%d\n",fn,obj_nr);
#endif
				dfp = fopen(fn, "r");
				if (dfp == NULL)
				{
					sprintf(e_msg,"%s: could not be opened\n",fn);
					perror(e_msg);
				}
				else
				{
					if ((ndfp = fopen(nfn, "w")) == NULL)
					{
						printf("Error writing to %s.\n", fn);
						continue;
					}
					printf("%s:\n", nfn);

#if defined(__DEBUG)
					printf("**************Converting....:%s->%s\n",fn,nfn);
#endif
					result = do_convert(dfp, ndfp, fn);
#if defined(__DEBUG)
					printf("**************Converted:%s->%s\n",fn,nfn);
#endif
					fclose(ndfp);
					fclose(dfp);
				}
#if defined(__DEBUG)
				//	sleep(1); //for readability...
#endif
			}
		}
		fclose(idxfp);
	}
	else
	{

		sprintf(fn,"%s",argv[1]);
		sprintf(nfn,"%s",argv[2]);

#if defined(__DEBUG)
		printf("In File/Out File:%s/%s\n",fn,nfn);
#endif
		dfp = fopen(fn, "r");
		if (dfp == NULL)
		{
			sprintf(e_msg,"%s: could not be opened\n",fn);
			perror(e_msg);
		}
		else
		{
			if ((ndfp = fopen(nfn, "w")) == NULL)
			{
				printf("Error writing to %s.\n", fn);
			}

#if defined(__DEBUG)
			printf("**************Converting....:%s->%s\n",fn,nfn);
#endif
			result = do_convert(dfp, ndfp, fn);
			printf("**************Converted:%s->%s\n",fn,nfn);
			fclose(ndfp);
			fclose(dfp);
		}
	}
	printf("Done!\n");
	return (0);
}
