/* ***********************************************************************
 *  File: wldconv.c                                                       *
 *  Usage: Convert a DangunMud .se file into a series of .wld  files      *
 *  																		 *
 ************************************************************************ */
#include "conf.h"
#include "sysdep.h"
#include "structs.h"
#include "dangun.h"
#include "Utils.h"

//#define __DEBUG	1

/**************************************************************************
 *  declarations of most of the 'global' variables                        *
 ************************************************************************ */
void delete_room(struct room_data * room)
{
	int	i;
	struct extra_descr_data * curr, * next;

	if (room->name)
		free(room->name);

	if (room->description)
		free(room->description);

	/* Free exits */
	for (i = 0; i < NUM_OF_DIRS; i++)
	{
		if (room->dir_option[i])
		{
			if (room->dir_option[i]->general_description)
				free(room->dir_option[i]->general_description);

			if (room->dir_option[i]->keyword)
				free(room->dir_option[i]->keyword);
		}

		free(room->dir_option[i]);
	}

	/* Free extra descriptions */
	for (curr = room->ex_description; curr; curr = next)
	{
		next = curr->next;

		if (curr->keyword)
			free(curr->keyword);

		if (curr->description)
			free(curr->description);

		free(curr);
	}
}

void dig_direction(struct room_data * room, int dir, const char * msg, int room_nr_to)
{
	CREATE(room->dir_option[dir], struct room_direction_data, 1);
	CREATE(room->dir_option[dir]->general_description, char, 256);

	//CREATE(room->dir_option[dir]->keyword, char, 256); // ¥ NULL 鲨 Ϸ?
	room->dir_option[dir]->keyword = NULL;
	room->dir_option[dir]->exit_info = 0;
	room->dir_option[dir]->key = -1;
	room->dir_option[dir]->to_room = room_nr_to;

	if (msg != NULL)
		strncpy(room->dir_option[dir]->general_description, msg, strlen(msg));
}

/* load the dangun rooms */
int load_room(FILE *fl, int virtual_nr, struct room_data * room)
{
	int t[10], i, retval,lines;
	char f_str[256],d_str[256];
	char flag_1[READ_SIZE],flag_2[READ_SIZE],flag_3[READ_SIZE];
	char *msg;
	char p_dir;
	//char flags[128], flags2[128], flags3[128], flags4[128];

	memset(t, 0, sizeof(t));
#if defined(__DEBUG)
	printf("Start of load_room:%d---------------------------------\n", virtual_nr);
#endif
	room->zone = virtual_nr / 100;
	room->number = virtual_nr;

	room->name = (char *) fread_str(fl, "read name");
	//printf("Room Name Stream:\n%s\n", room->name);
	room->description = (char *) fread_str(fl, "read desc");
	//printf("Room Description Stream:\n%s\n", room->description);

	lines = get_line(fl, flag_1);
	//printf("lines(%d):%s\n", lines, flag_1);
	retval = sscanf(flag_1, "%d\n", t+0);

	lines = get_line(fl, flag_2);
	//printf("lines(%d):%s\n", lines, flag_2);
	retval = sscanf(flag_2, "%d %d %d\n", t+1,t+2,t+3);

	lines = get_line(fl, flag_3);
	//printf("lines(%d):%s\n", lines, flag_3);
	retval = sscanf(flag_3, "%d",t+4);

	// t[0]
	if (t[0] == 0)	// 0̸ ο 
		SET_BIT_AR(room->room_flags, ROOM_DARK);

	// t[1]
	/* proccessing sector type*/	
	room->sector_type = 0; // INDOORS

	switch (t[1])
	{
		case -6: //
			room->sector_type = SECT_UNDERWATER;
			break;
		case -5: //
			room->sector_type = SECT_WATER_SWIM;
			break;
		case -4: //ȭ
			room->sector_type = SECT_VOLCANO;
			break;
		case -3: //ϴ
			room->sector_type = SECT_FLYING;
			break;
		case -2: //
			room->sector_type = SECT_WATER_NOSWIM;
			break;
		case -1: //	(roomflag: death trap)
			SET_BIT_AR(room->room_flags, ROOM_DEATH);
			room->sector_type = SECT_FIELD;
			break;
		case 0: // 
			room->sector_type = SECT_ROAD;
			break;
		case 1: // 
			room->sector_type = SECT_HILLS;
			break;
		case 2: // ǳ
			room->sector_type = SECT_INSIDE;
			break;
		case 3: // 
			room->sector_type = SECT_SWAMP;
			break;
		case 4: // 
			room->sector_type = SECT_VALLEY;
			break;
		case 5: // 縷
			room->sector_type = SECT_DESERT;
			break;
		case 6: // 
			room->sector_type = SECT_FOREST;
			break;
		case 7: // 
			room->sector_type = SECT_MOUNTAIN;
			break;
		case 8: // 
			room->sector_type = SECT_FIELD;
			break;

		default:
			printf("Unknown sector type %d, +%d %s\n", t[1], virtual_nr, room->name);
			break;
	}

	// t[2]
	room->moveloss = t[2];

	// t[3]
	/* Proccessing room flag */
	room->room_flags[0] = 0;
	room->room_flags[1] = 0;
	room->room_flags[2] = 0;
	room->room_flags[3] = 0;

	room->room_affects = t[4];

	if (room->room_affects != 0)
		printf("   %d AFFECT %d\n", virtual_nr, room->room_affects);

	if (IS_SET(t[3], DR_INDOORS)) SET_BIT_AR(room->room_flags, ROOM_INDOORS); // Ϲݹ	⺻
	if (IS_SET(t[3], DR_PRIVATE) && IS_SET(t[3], DR_NOMOB))	SET_BIT_AR(room->room_flags, ROOM_NOMOB); // 	 ( )
	if (IS_SET(t[3], DR_HOLY))
	{
		//printf("romm_flag:DR_HOLY\n");
		SET_BIT_AR(room->room_flags, ROOM_PEACEFUL);	//  (ȸ+a), 
		SET_BIT_AR(room->room_flags, ROOM_NOMAGIC);				
	}
	//if (IS_SET(t[3], DR_NOEXIT))
	//	printf("   %d NO EXIT\n", virtual_nr);

	if (IS_SET(t[3], DR_PREACT)) SET_BIT_AR(room->room_flags, ROOM_THUNDER);		// гƼ	ݽ HP-50%
	if (IS_SET(t[3], DR_PKILL)) SET_BIT_AR(room->room_flags, ROOM_PK);				// PK			
	if (IS_SET(t[3], DR_NOEXIT)) SET_BIT_AR(room->room_flags, ROOM_HIDDEN_EXIT);	// ⱸ	ⱸ Ⱥ(̵)
	if (IS_SET(t[3], DR_NONORTH)) SET_BIT_AR(room->room_flags, ROOM_HIDDEN_NORTH);	// ʾȺ ⱸǥ	FIXME
	if (IS_SET(t[3], DR_NOEAST)) SET_BIT_AR(room->room_flags, ROOM_HIDDEN_EAST);	// ʾȺ ⱸǥ	FIXME
	if (IS_SET(t[3], DR_NOWEST)) SET_BIT_AR(room->room_flags, ROOM_HIDDEN_WEST);	// ʾȺ ⱸǥ	FIXME
	if (IS_SET(t[3], DR_NOSOUTH)) SET_BIT_AR(room->room_flags, ROOM_HIDDEN_SOUTH);	// ʾȺ ⱸǥ	FIXME
	if (IS_SET(t[3], DR_NOUP)) SET_BIT_AR(room->room_flags, ROOM_HIDDEN_UP);		// ʾȺ ⱸǥ	FIXME
	if (IS_SET(t[3], DR_NODOWN)) SET_BIT_AR(room->room_flags, ROOM_HIDDEN_DOWN);	// ʾȺ ⱸǥ	FIXME
	if (IS_SET(t[3], DR_QUIT)) SET_BIT_AR(room->room_flags, ROOM_RENTABLE);			// ñ		ϰ 
	if (IS_SET(t[3], DR_NVEHICLE)) SET_BIT_AR(room->room_flags, ROOM_NOVEHICLE);	// Ÿ	  Ÿ  FIXME
	if (IS_SET(t[3], DR_NOPEEK)) SET_BIT_AR(room->room_flags, ROOM_NOPEEK);			// 	 ȵ FIXME

	sprintf(f_str, "RoomFlag:(%d:%d:%d:%d) SectorType:%d\n", 
			room->room_flags[0],
			room->room_flags[1], 
			room->room_flags[2],
			room->room_flags[3],
			room->sector_type);
#if defined(__DEBUG)
	puts(f_str);
#endif
	room->func = NULL;
	room->contents = NULL;
	room->people = NULL;
	room->light = t[0];	/* light sources */
	room->timed = -1;

	for (i = 0; i < NUM_OF_DIRS; i++)
		room->dir_option[i] = NULL;

	room->ex_description = NULL;

	for (;;)
	{
		lines = get_line(fl, d_str);
		d_str[strlen(d_str)] = '\0';

#if defined(__DEBUG)
		printf("lines(%d):%s\n", lines, d_str);
#endif
		retval = sscanf(d_str, "%c %d\n", &p_dir, t + 5);

		if (p_dir == 'E')
		{
#if defined(__DEBUG)
			printf("End of load_room:%d---------------------------------\n", virtual_nr);
#endif
			return 0;
		}

		msg = (char *) fread_str(fl, "read msg");
#if defined(__DEBUG)
		printf("Stream:\n%s\n",msg);
#endif
		switch (p_dir)
		{
			case 'n': dig_direction(room, NORTH, msg, t[5]); break;
			case 'e': dig_direction(room, EAST, msg, t[5]); break;
			case 's': dig_direction(room, SOUTH, msg, t[5]); break;
			case 'w': dig_direction(room, WEST, msg, t[5]); break;
			case 'u': dig_direction(room, UP, msg, t[5]); break;
			case 'd': dig_direction(room, DOWN, msg, t[5]); break;
			default: 
					  printf("Unknown direction %c +%d %s\n", p_dir, virtual_nr, room->name);
					  free(msg);
					  exit(1);
					  break;
		}

		free(msg);
	}
}

static int do_convert(FILE * dan_f, FILE * newdan_f, char * filename)
{
	struct room_data * room;
	char rhead[READ_SIZE];
	char r_str[MAX_STRING_LENGTH], d_str[1024];
	char r_desc[512], r_key[32];

	int i, virtual_nr, lines;

	for (;;)
	{
		lines = get_line(dan_f, rhead);
		rhead[strlen(rhead)] = '\0';
#if defined(__DEBUG)
		printf("lines(%d):%s\n",lines, rhead);
#endif
		if (rhead[0] == '+')
		{
			char * flag1, * flag2, * flag3, * flag4, * affect_flag;

			/* start room file */
			sscanf(rhead, "+%d\n", &virtual_nr);

			CREATE(room, struct room_data, 1);
			load_room(dan_f, virtual_nr, room);

			// to_ascii ޸𸮸 Ҵϰ  ֱ  ݵ
			// ӽú  free ־ Ѵ. ( ʹ  ϰ
			// Ѿ  ְ;;)
			flag1 = to_ascii(room->room_flags[0]);
			flag2 = to_ascii(room->room_flags[1]);
			flag3 = to_ascii(room->room_flags[2]);
			flag4 = to_ascii(room->room_flags[3]);
			affect_flag = to_ascii(room->room_affects);

			/* Build a buffer ready to save */
			sprintf(r_str, "#%d\n%s~\n%s~\n%d %s %s %s %s %d %s\n",
					virtual_nr, 
					room->name ? room->name : "̿ϼ", 
					room->description, 
					room->moveloss, 
					flag1, 
					flag2, 
					flag3, 
					flag4, 
					room->sector_type,
					affect_flag);

			free(flag1);
			free(flag2);
			free(flag3);
			free(flag4);
			free(affect_flag);

#if defined(__DEBUG)
			printf("%s",r_str);
#endif
			/* Save this section */
			fputs(r_str, newdan_f);

			/* Handle exits */
			for (i = 0; i < NUM_OF_DIRS; i++)
			{ 
				if (!room->dir_option[i])
					continue;

				{ 
					int temp_door_flag = 0;

					if (room->dir_option[i]->general_description)
						strcpy(r_desc, room->dir_option[i]->general_description);

					//     ܱ  .se Ͽ ãƳ 
					/*
					if (IS_SET(room->dir_option[i]->exit_info, EX_ISDOOR))
					{ 
						if (IS_SET(room->dir_option[i]->exit_info, EX_PICKPROOF))
							temp_door_flag = 2;
						else
							temp_door_flag = 1;
					}
					else
						temp_door_flag = 0;
					 */

					/* Check for keywords */
					// keyword  ܱ  .se Ͽ ãƳ 
					//if (room->dir_option[i]->keyword)
					//{
					//   strcpy(r_key, room->dir_option[i]->keyword);
					//}
					*r_key = '\0';

					/* Ok, now build a buffer to output to file */
					sprintf(d_str, "D%d\n%s~\n%s~\n%d %i %i\n", 
							i, 
							r_desc, 
							r_key,
							temp_door_flag,
							(sh_int) room->dir_option[i]->key,
							room->dir_option[i]->to_room);
					/* Save this door */
#if defined(__DEBUG)
					puts(d_str);
#endif
					fputs(d_str, newdan_f);
				}
			}

			// ܱ  .se Ͽ ex_description   .
			/*	
			if (room->ex_description)
			{ 
				struct extra_descr_data * ex_desc;

				for (ex_desc = room->ex_description; ex_desc; ex_desc = ex_desc->next)
				{ 
					// Home straight, just deal with extras descriptions..
					strcpy(e_desc, ex_desc->description);
					sprintf(e_str, "E\n%s~\n%s~\n", ex_desc->keyword, e_desc);
					fputs(e_str, newdan_f);
				}
			}
			*/
			fprintf(newdan_f, "S\n");
			delete_room(room); 
		}
		else
		{
			if (rhead[0] == '@')
			{
				/* EOF */
				fprintf(newdan_f, "$~\n");
				break;
			}
			else
			{
				printf("%s: Wrong format detected, conversion aborted!\n", filename);
				return (1);
			}
		}
	}

	fclose(newdan_f);
#if defined(__DEBUG)
	printf("The End of conver\n");
#endif
	return (0);
}

int wldconv(int argc, char ** argv)
{
	FILE *idxfp,*dfp, *ndfp, *nidxfp;
	char fn[256], nfn[256];
	char idx_f[READ_SIZE], e_msg[256];
	int result, lines, zone_nr, retval, idx_flag;

	if (argc < 2)
	{
		printf("Usage: %s [Index] [<infile.se> <outfile.wld>]\n", *argv);
		exit(1);
	}

	if (!strcmp(argv[1], "Index"))
	{
		printf("Converting each file \n");
		strcpy(fn, argv[1]);
		idx_flag = 1;
	}
	else
		strcpy(fn, "Index");

	if (idx_flag == 1)
	{
		if (!(nidxfp = fopen("Index.wld.new", "w")))
		{
			strcat(e_msg, " Index.wld.new: could not be opened");
			perror(e_msg);
			return (1);
		}

		if (!(idxfp = fopen("Index", "r")))
		{
			strcat(e_msg, " Index: could not be opened");
			perror(e_msg);
			fclose(nidxfp);
			return (1);
		}
		else
		{
			for (;;)
			{
				lines = get_line(idxfp, idx_f);
				idx_f[strlen(idx_f)] = '\0';
#if defined(__DEBUG)
				printf("lines(%d): %s\n", lines, idx_f);
#endif
				if (*idx_f == '$')
					break;

				if (!isalnum(*idx_f))
					continue;

				retval = sscanf(idx_f, "%s %d\n", fn, &zone_nr);
#if defined(__DEBUG)
				printf("fn/zone_nr:%s/%d\n",fn,zone_nr);
#endif
				if (!(dfp = fopen(fn, "r")))
				{
					sprintf(e_msg,"%s: could not be opened\n",fn);
					perror(e_msg);
				}
				else
				{
					*strrchr(fn, '.') = '\0';
					sprintf(nfn, "%d_%s.wld", zone_nr, fn);
					fprintf(nidxfp, "%s\n", nfn);

					if (!(ndfp = fopen(nfn, "w")))
					{
						printf("Error writing to %s.\n", fn);
						continue;
					}
					printf("%s:\n", nfn);

#if defined(__DEBUG)
					printf("**************Converting....:%s->%s\n",fn,nfn);
#endif
					result = do_convert(dfp, ndfp, fn);
#if defined(__DEBUG)
					printf("**************Converted:%s->%s\n",fn,nfn);
#endif
					fclose(ndfp);
					fclose(dfp);
				}
#if defined(__DEBUG)
				//	sleep(1); //for readability...
#endif
			}
		}
		fclose(nidxfp);
		fclose(idxfp);
	}
	else
	{
		sprintf(fn,"%s",argv[1]);
		sprintf(nfn,"%s",argv[2]);

#if defined(__DEBUG)
		printf("In File/Out File:%s/%s\n",fn,nfn);
#endif
		if (!(dfp = fopen(fn, "r")))
		{
			sprintf(e_msg,"%s: could not be opened\n",fn);
			perror(e_msg);
		}
		else
		{
			if (!(ndfp = fopen(nfn, "w")))
				printf("Error writing to %s.\n", fn);
			else
			{
				printf("%s:\n", nfn);

#if defined(__DEBUG)
				printf("**************Converting....:%s->%s\n",fn,nfn);
#endif
				result = do_convert(dfp, ndfp, fn);
#if defined(__DEBUG)
				printf("**************Converted:%s->%s\n",fn,nfn);
#endif
				fclose(ndfp);
			}
			fclose(dfp);
		}
	}

	printf("Done!\n");
	return (0);
}
