/* ************************************************************************ *
 *  File: zonconv.c                                                          *
 *  Usage: Convert DangunMud's .zon files into series of circle's .zon files *
 *  																			*
 *************************************************************************** */
#include "conf.h"
#include "sysdep.h"
#include "structs.h"
#include "dangun.h"
#include "Utils.h"

#define	MAX_CMDS		12288

struct reset_com
{
	char	command;	/* current command                      */
	bool	if_flag;	/* if TRUE: exe only if preceding exe'd */
	int		arg1;		/*                                      */
	int		arg2;		/* Arguments to the command             */
	int		arg3;		/*                                      */
	int		arg4;		/* percentages variable					*/
	int		line;		/* line number this command appears on  */
	char *	sarg1;		/* string argument                      */
	char *	sarg2;		/* string argument                      */

	/* 
	 *  Commands:              *
	 *  'M': Read a mobile     *
	 *  'O': Read an object    *
	 *  'G': Give obj to mob   *
	 *  'P': Put obj in obj    *
	 *  'G': Obj to char       *
	 *  'E': Obj to char equip *
	 *  'D': Set state of door *
	 *  'T': Trigger command   *
	 *  'V': Assign a variable *
	 */
};

/* zone definition structure. for the 'zone-table'   */
struct zone_data
{
	char	* name;		    // name of this zone
	char	* builders;     // namelist of builders allowed to modify this zone.
	int	lifespan;           // how long between resets (minutes)
	int	age;                // current age of this zone (minutes)
	room_vnum bot;          // starting room number for this zone
	room_vnum top;          // upper limit for rooms in this zone

	int	reset_mode;         // conditions for reset (see below)
	zone_vnum number;	    // virtual number of this zone
	struct reset_com * cmd; // command table for reset

	/*
	 * Reset mode:
	 *   0: Don't reset, and don't update age.
	 *   1: Reset if no PC's are located in zone.
	 *   2: Just reset.
	 */
};

/**************************************************************************
 *  declarations of most of the 'global' variables                         *
 ************************************************************************ */
struct zone_data *wzon = NULL; /* arry of zons */
int cmd_cnt, d_flag = 0;

/**************************************************************************
 * declarations of local function 																				 *
 ************************************************************************ */
int load_zon(FILE *fl,int room_nr,char *mhead);
void write_zon(FILE *fp);
int do_convert(FILE * dan_f, FILE * newdan_f, char *filename);

int load_zon(FILE *fl, int room_nr, char *mhead)
{
	int retval;
	int d_cmd;
	char d_name[256];
	int	d_dir, d_door, d_key, d_where;
	int d_num, d_total, d_pct; 
	char * c_str;

#if defined(__DEBUG)
	printf("Start of load_room(%d)---------------------------------\n",room_nr);
#endif
	c_str = mhead;
	skip_spaces(&c_str);

	if (!*c_str)
		return 0;

#if defined(__DEBUG)
	printf("mhead: %s\n", mhead);
#endif
	if (c_str[0] == 'X') //    (  ׷  -_-)
	{
		retval = sscanf(c_str, "%c %s %d %d %d %d\n", &d_cmd, d_name, &d_dir, &d_door, &d_key, &d_where); 
		wzon->cmd[cmd_cnt].command = 'D';
		wzon->cmd[cmd_cnt].if_flag = 1;
		wzon->cmd[cmd_cnt].line = cmd_cnt;
		//wzon->cmd[cmd_cnt].sarg1 = strdup(d_name);
		wzon->cmd[cmd_cnt].arg1 = room_nr;

		switch (d_dir)
		{
			case 0:
				//Ÿ, (,  ,  Circle   ִ)
				if (IS_SET(d_door, DR_DOOR_ENTER))
					wzon->cmd[cmd_cnt].arg2 = INDIR;
				else if (IS_SET(d_door, DR_DOOR_LEAVE))
					wzon->cmd[cmd_cnt].arg2 = OUTDIR;
				break;

			case 1:
				wzon->cmd[cmd_cnt].arg2 = EAST; //east
				break;

			case 2: 
				wzon->cmd[cmd_cnt].arg2 = WEST; //west
				break;

			case 3: 
				wzon->cmd[cmd_cnt].arg2 = SOUTH; //south
				break;

			case 4:
				wzon->cmd[cmd_cnt].arg2 = NORTH; //north
				break;

			case 5: 
				wzon->cmd[cmd_cnt].arg2 = UP; //up
				break;

			case 6: 
				wzon->cmd[cmd_cnt].arg2 = DOWN; //down
				break;

			default:
				printf("  Unknown direction %d!!! room %d\n", d_dir, room_nr);
				exit(1);
		}

		/*
		// Dangun Door state	
		//    1       ( ,  ° ƴҰ ְ   ,
		//                   ǿ  ڹ踦 ų Ͽ  )
		//	  2 XX ڹ   (,   ٷδ ƹ   ÷,   ִ  )
		//	  4 XX  (ӿ ұϰ ڹ ۵ -_- Ʈ  ʾƾ )
		//	  8 乮      (谡 ְ  ,  ƴ 쿡 ڹ )
		//   16       (ڹ   )
		//   32 ¹  (   Ƿ  ⿡ Ѵ)
		//   64 ¹    (   Ƿ  ⿡ Ѵ)
		// 1024 ڵݱ⹮  (ش     , 󰡱   ?)
		// 2048   ( )
		// 8192 Ȯι  ( ־߸   )
		*/
		//if (d_door && IS_SET(d_door, DR_DOOR_OPEN) && !IS_SET(d_door, DR_DOOR_LIMIT))
		//	 printf("  !!!!!!!!!!!!!!! %d flag set\n", room_nr);

		//
		//  ε,   :
		//           :   , ѿ    
		//   빮  :  ׷   ִµ ,  ȵ
		//   ǳϹ        :  ִ°?  ִµ ׳ 
		//	               :   ɾ  , ־   Ƿ
		//
		// Ʈ:
		//   DOOR_OPEN ° ƴ   CLOSED· .
		//      ε OPEN  (빮)  ¸ ش.
		//
		switch (d_door)
		{
			case 1: 
				wzon->cmd[cmd_cnt].arg3 = 0;  // 
				break;

			case 2: 
				wzon->cmd[cmd_cnt].arg3 = 1;  // ڹ
				break;

			case 4: 
				wzon->cmd[cmd_cnt].arg3 = 1;  // 
				break;

			case 8:
				wzon->cmd[cmd_cnt].arg3 = 2;  // 乮
				break;

			case 9: 
				wzon->cmd[cmd_cnt].arg3 = 1;  // +乮: 빮.....
				break;

			case 12: 
				wzon->cmd[cmd_cnt].arg3 = 2;  // +乮
				break;

			case 24: 
				wzon->cmd[cmd_cnt].arg3 = 2;  // 乮+: ..
				break;

			case 32: 
				wzon->cmd[cmd_cnt].arg3 = 2;  //  :,Ｚ...
				break;

			case 33: 
				wzon->cmd[cmd_cnt].arg3 = 0;  // +: ,Ｚ...
				break;

			case 56: 
				wzon->cmd[cmd_cnt].arg3 = 2;  // 乮++
				break;

			case 64: 
				wzon->cmd[cmd_cnt].arg3 = 0;  // ¹
				break;

			case 88: 
				wzon->cmd[cmd_cnt].arg3 = 0;  // 24+¹
				break;

			case 2049: // (1 << 0) | (1 << 12)
				wzon->cmd[cmd_cnt].arg3 = 0;  // +: 
				break;

			case 1024:
				wzon->cmd[cmd_cnt].arg3 = 0;  // ڵݱ⹮
				break;

			default:
				printf("Room %d Door state? [%d] (0:open,1:close,2:close&lock)\n", room_nr, d_door);
				//getchar();
				break;
		}
	}
	else
	{
		retval = sscanf(c_str, "%c %d %d %d\n", &d_cmd, &d_num, &d_total, &d_pct);
		wzon->cmd[cmd_cnt].line = cmd_cnt;
		wzon->cmd[cmd_cnt].arg1 = d_num;
		wzon->cmd[cmd_cnt].arg2 = d_total;
		wzon->cmd[cmd_cnt].arg4 = 0; // arg4 Ȯ ϸ, 0̸ 100%, 100̸ 99%ȳ ȴ. (99  99% ȳ)

		switch (c_str[0])
		{
			case 'M':
				// M ȣ Ѱ
				wzon->cmd[cmd_cnt].if_flag = 0;
				wzon->cmd[cmd_cnt].command = 'M';
				wzon->cmd[cmd_cnt].arg3 = room_nr;
				wzon->cmd[cmd_cnt].arg2 *= 10;	// Ѱ ǹ̰ Ƿ *10 Ͽ    ְ 
				break;

			case 'N':
				// N ȣ üѰ Ѱ(  )
				wzon->cmd[cmd_cnt].if_flag = 0;
				wzon->cmd[cmd_cnt].command = 'M';
				wzon->cmd[cmd_cnt].arg3 = room_nr;
				break;

			case 'G':
				// G ۹ȣ üѰ Ȯ
				wzon->cmd[cmd_cnt].if_flag = 1;
				wzon->cmd[cmd_cnt].command = 'G';	// 
				wzon->cmd[cmd_cnt].arg4 = 100 - d_pct;
				break;

			case 'W': // WEAR,    
				// W ۹ȣ üѰ Ȯ
				wzon->cmd[cmd_cnt].if_flag = 1;
				wzon->cmd[cmd_cnt].command = 'E';	// 
				wzon->cmd[cmd_cnt].arg3 = 12;		// wear about body
				wzon->cmd[cmd_cnt].arg4 = 100 - d_pct;
				break;

			case 'D': // WIELD
				// D ۹ȣ üѰ Ȯ
				if (d_flag == 0)
				{
					wzon->cmd[cmd_cnt].if_flag = 1;
					wzon->cmd[cmd_cnt].command = 'E';	// 
					wzon->cmd[cmd_cnt].arg3 = 16;		// wield 
					d_flag++;
				}
				else
				{
					wzon->cmd[cmd_cnt].if_flag = 1;
					wzon->cmd[cmd_cnt].command = 'G';	// ߺ  
					d_flag = 0;
				}
				wzon->cmd[cmd_cnt].arg4 = 100 - d_pct;
				break;

			case 'H': // HOLD
				// H ۹ȣ üѰ Ȯ
				wzon->cmd[cmd_cnt].if_flag = 1;
				wzon->cmd[cmd_cnt].command = 'E';
				wzon->cmd[cmd_cnt].arg3 = 17;			// held
				break;

			case 'I':
				// I ۹ȣ üѰ Ȯ
				wzon->cmd[cmd_cnt].if_flag = 0;
				wzon->cmd[cmd_cnt].command = 'O';		// Object  ġ
				wzon->cmd[cmd_cnt].arg3 = room_nr;		// ̳ΰ ļ I Option
				wzon->cmd[cmd_cnt].arg4 = 100 - d_pct;
				break;

			case 'P':
				// P ۹ȣ üѰ Ȯ
				wzon->cmd[cmd_cnt].if_flag = 1;
				wzon->cmd[cmd_cnt].command = 'P';		// Object in Object type
				//wzon->cmd[cmd_cnt].arg3 = ??; // Ʈ ȣ ־  FIXME FIXME!!
				wzon->cmd[cmd_cnt].arg4 = 100 - d_pct;
				break;

			default:
				printf("Unknown command '%c' line %s\n", c_str[0], c_str);
				getchar();
				break;
		}	 
	}

	if (++cmd_cnt >= MAX_CMDS)
	{
		printf("ERROR!! CMD COUNT OVERFLOW %d!!!!!!!!!!!!!\n", cmd_cnt);
		exit(1);
	}
	
#if defined(__DEBUG)
	printf("End of load_room(%d)---------------------------------\n",room_nr);
#endif
	return 0;
}

void write_zon(FILE *fp)
{
	int subcmd, arg1, arg2, arg3, arg4;
	char * comment = NULL;

	arg1 = arg2 = arg3 = arg4 = -1;

#if defined(__DEBUG)
	printf("wzon->number:%d\n",wzon->number);
	printf("wzon->builders:%s\n",wzon->builders);
	printf("wzon->name:%s\n",wzon->name);
	printf("wzon->bot:%d\n",wzon->bot);
	printf("wzon->top:%d\n",wzon->top);
	printf("wzon->reset_mode:%d\n",wzon->reset_mode);
#endif
	/*
	 * Print zone header to file	
	 */

	fprintf(fp, "#%d\n"
				"%s~\n"
			"%s~\n"
			"%d %d %d %d\n",
			wzon->number,
			(wzon->builders) ? wzon->builders : "*",
			(wzon->name) ? wzon->name : " ",
			wzon->bot,
			wzon->top,
			wzon->lifespan,
			wzon->reset_mode);

	CREATE(comment, char, 128);

	for (subcmd = 0; subcmd < cmd_cnt; subcmd++)
	{
#if defined(__DEBUG)
		printf("command:%c\n", wzon->cmd[subcmd].command);
#endif
		switch (wzon->cmd[subcmd].command)
		{
			case 'M':
				arg1 = wzon->cmd[subcmd].arg1;
				arg2 = wzon->cmd[subcmd].arg2;
				arg3 = wzon->cmd[subcmd].arg3;
				arg4 = wzon->cmd[subcmd].arg4;
				sprintf(comment, "Mob[%d]",arg1);
				break;

			case 'O':
				arg1 = wzon->cmd[subcmd].arg1;
				arg2 = wzon->cmd[subcmd].arg2;
				arg3 = wzon->cmd[subcmd].arg3;
				arg4 = wzon->cmd[subcmd].arg4;
				sprintf(comment, "Object[%d]",arg1);
				break;

			case 'G':
				arg1 = wzon->cmd[subcmd].arg1;
				arg2 = wzon->cmd[subcmd].arg2;
				arg3 = -1;
				arg4 = wzon->cmd[subcmd].arg4;
				sprintf(comment, "Object[%d] is given....",arg1);
				break;

			case 'E':
				arg1 = wzon->cmd[subcmd].arg1;
				arg2 = wzon->cmd[subcmd].arg2;
				arg3 = wzon->cmd[subcmd].arg3;
				arg4 = wzon->cmd[subcmd].arg4;
				sprintf(comment, "Object[%d] is equipped....",arg1);
				break;

			case 'P':
				arg1 = wzon->cmd[subcmd].arg1;
				arg2 = wzon->cmd[subcmd].arg2;
				arg3 = wzon->cmd[subcmd].arg3;
				arg4 = wzon->cmd[subcmd].arg4;
				sprintf(comment, "Object in Object[%d][%d[",arg4,arg1);
				break;

			case 'D':
				arg1 = wzon->cmd[subcmd].arg1;
				arg2 = wzon->cmd[subcmd].arg2;
				arg3 = wzon->cmd[subcmd].arg3;
				sprintf(comment, "Door[%d] is located....",arg1);
				break;

			default:
				printf("SYSERR: Unknown cmd '%c' - NOT saving", wzon->cmd[subcmd].command);
				getchar();
		}

		fprintf(fp, "%c %d %d %d %d %d \t(%s)\n",
				wzon->cmd[subcmd].command, wzon->cmd[subcmd].if_flag, arg1, arg2, arg3, arg4, comment);
	}

	free(comment);
}

static int do_convert(FILE * dan_f, FILE * newdan_f, char *filename)
{
	char mhead[128];
	int virtual_nr, lines, zon_nr,regen_tick, room_nr;
	int last_room;
	int next_room = 1;

	CREATE(wzon, struct zone_data, 1);

	cmd_cnt = 0;
	zon_nr = 0; //  ...

#if defined(__DEBUG)
	printf(" Start of zon=========================================(zon:%d)\n",zon_nr);
#endif
	lines = get_line(dan_f,mhead); //+zone	
	mhead[strlen(mhead)]='\0';
	sscanf(mhead, "+%d\n", &virtual_nr);
#if defined(__DEBUG)
	printf("lines(%d):%s\n",lines,mhead);
#endif
	lines=get_line(dan_f,mhead);	
	mhead[strlen(mhead)]='\0';
	sscanf(mhead, "%d\n", &regen_tick);
#if defined(__DEBUG)
	printf("lines(%d):%d\n",lines,regen_tick);
#endif
	wzon->number = virtual_nr;
	wzon->name = strdup(filename);
	wzon->lifespan = regen_tick;
	wzon->reset_mode = 2;

	CREATE(wzon->cmd, struct reset_com, MAX_CMDS);

	wzon->bot = virtual_nr * 100;
	wzon->top = wzon->bot + 512;

	for (;;)
	{
		lines = get_line(dan_f, mhead);	
		mhead[strlen(mhead)] = '\0';

#if defined(__DEBUG)
		printf("lines(%d),room(%d):%s\n",lines,room_nr,mhead);
#endif
		if (isdigit(*mhead))
		{
			sscanf(mhead, "%d\n", &room_nr);
			last_room = room_nr;
			d_flag = 0;
		}					
		else if (isspace(mhead[0]))
		{
			/* start zon file */
			next_room = load_zon(dan_f,room_nr,mhead);
		}
		else if (mhead[0] == '@')
		{
			/* EOF */
			write_zon(newdan_f);
			fprintf(newdan_f, "S\n$\n");

			if (wzon->name)
			{
				free(wzon->name);
				wzon->name = NULL;
			}

			if (wzon->cmd)
			{
				free(wzon->cmd);
				wzon->cmd = NULL;
			}

			free(wzon);
			wzon = NULL;

			cmd_cnt = 0;
#if defined(__DEBUG)
			printf(" End of zon===========================================(zon:%d)\n",zon_nr);
#endif
			return (0);
		} 
		else
		{
			printf("Input File Error: Symentics is not correct\n");
			exit(1);
			return (1);
		}
		zon_nr++;
	}
}

int zonconv(int argc, char ** argv)
{
	FILE *idxfp,*dfp, *ndfp;
	char fn[256], nfn[256];
	char idx_f[256],e_msg[256];
	int result, lines,zon_nr,retval,idx_flag;

	if (argc < 2)
	{
		printf("Usage: zonconv [Index] [<infile.mul> <outfile.zon>]\n");
		exit(1);
	}

	if (!strcmp(argv[1], "Index"))
	{
		printf("Converting each file \n");
		strcpy(fn, argv[1]);
		idx_flag = 1;
	}

	if (idx_flag == 1)
	{
		idxfp = fopen("Index", "r");

		if (idxfp == NULL)
		{
			strcat(e_msg, " Index: could not be opened");
			perror(e_msg);
			return(1);
		}
		else
		{
			for (;;)
			{
				lines = get_line(idxfp,idx_f);
				idx_f[strlen(idx_f)]='\0';
#if defined(__DEBUG)
				printf("lines(%d):%s\n",lines,idx_f);
#endif
				if (idx_f[0]=='$')
				{
					break;
				}

				retval= sscanf(idx_f,"%s %d\n",fn,&zon_nr);

				sprintf(nfn,"%d.zon",zon_nr);

				printf("%s %d -> ", fn, zon_nr);

				dfp = fopen(fn, "r");
				if (dfp == NULL)
				{
					sprintf(e_msg,"%s: could not be opened\n",fn);
					perror(e_msg);
				}
				else
				{
					if ((ndfp = fopen(nfn, "w")) == NULL)
					{
						printf("Error writing to %s.\n", fn);
						continue;
					}
					printf("%s:\n", nfn);

#if defined(__DEBUG)
					printf("**************Converting....:%s->%s\n",fn,nfn);
#endif
					result = do_convert(dfp, ndfp, fn);
#if defined(__DEBUG)
					printf("**************Converted:%s->%s\n",fn,nfn);
#endif
					fclose(ndfp);
					fclose(dfp);
				}
#if defined(__DEBUG)
				//	sleep(1); //for readability...
#endif
			}
		}
		fclose(idxfp);
	}
	else
	{
		sprintf(fn, "%s",argv[1]);
		sprintf(nfn, "%s",argv[2]);

#if defined(__DEBUG)
		printf("In File/Out File:%s/%s\n",fn,nfn);
#endif
		dfp = fopen(fn, "r");
		if (dfp == NULL)
		{
			sprintf(e_msg,"%s: could not be opened\n",fn);
			perror(e_msg);
		}
		else
		{
			if ((ndfp = fopen(nfn, "w")) == NULL)
			{
				printf("Error writing to %s.\n", fn);
			}

#if defined(__DEBUG)
			printf("**************Converting....:%s->%s\n",fn,nfn);
#endif
			result = do_convert(dfp, ndfp, fn);
			printf("**************Converted:%s->%s\n",fn,nfn);
			fclose(ndfp);
			fclose(dfp);
		}
	}
	printf("Done!\n");
	return (0);
}
